/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef CMDLINE_H
#define CMDLINE_H

#include <kcmdlineargs.h>

#include "common/log.h"
#include "common/qflags.h"
#include "progs/base/prog_list.h"
#include "devices/base/hex_buffer.h"

class CommandLine
{
public:
  static const KCmdLineOptions OPTIONS[];
  static int process();

  enum ExitCodes { OK = 0, ARG_ERROR = -1, NOT_SUPPORTED_ERROR = -2,
                   FILE_ERROR = -3, PROG_ERROR = -4 };
  enum CommandProperty { NoProperty = 0, NeedProgrammer = 1, InputHex = 2,
                         OutputHex = 4, NeedDevice = 8 };
  Q_DECLARE_FLAGS(CommandProperties, CommandProperty)
  enum Command { Connect = 0, Run, Program, Verify, Read, Erase, BlankCheck,
                 Checksum, UploadFirmware, Nb_Commands };
  struct CommandData {
    const char *name;
    CommandProperties properties;
  };
  static const CommandData COMMAND_DATA[Nb_Commands];

  static KCmdLineArgs *_args;
  static const Programmer::Group *_group;
  static Command _command;
  static QString _device;
  static HexBuffer::Format _format;
  static Log::ConsoleView *_log;

  static int errorExit(const QString &message, int code);
  static int commandList();
  static int formatList();
  static int programmerList();
  static int deviceList();
};
Q_DECLARE_OPERATORS_FOR_FLAGS(CommandLine::CommandProperties)

#endif
