/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "group.h"

#include "devices/base/generic_device.h"

//-----------------------------------------------------------------------------
void Group::addDevice(const Device::Data *data)
{
  _devices[data->name()] = data;
}

const Device::Data *Group::deviceData(const QString &device) const
{
  if ( !_devices.contains(device) ) return 0;
  return _devices[device];
}

QStringList Group::supportedDevices() const
{
  QStringList list;
  QMap<QString, const Device::Data *>::const_iterator it;
  for (it=_devices.begin(); it!=_devices.end(); ++it)
    list += it.key();
  return list;
}

void Group::reinitSupported()
{
  _devices.clear();
  initSupported();
}

//-----------------------------------------------------------------------------
void ListerBase::addGroup(Group *group)
{
  group->init();
  _groups.append(group);
}

const Group *ListerBase::group(const QString &name) const
{
  for (uint i=0; i<nbGroups(); i++) {
    if ( _groups[i]->name()!=name ) continue;
    return _groups[i];
  }
  return 0;
}

bool ListerBase::isSupported(const QString &device) const
{
  for (uint i=0; i<nbGroups(); i++)
    if ( _groups[i]->isSupported(device) ) return true;
  return false;
}

bool ListerBase::isSupported(uint k, const QString &device) const
{
  if ( k!=nbGroups() ) return _groups[k]->isSupported(device);
  return isSupported(device);
}

QStringList ListerBase::supportedDevices() const
{
  QStringList names;
  for (uint i=0; i<nbGroups(); i++) {
    QStringList list = _groups[i]->supportedDevices();
    for (uint k=0; k<list.count(); k++)
      if ( names.find(list[k])==names.end() ) names.append(list[k]);
  }
  return names;
}

QStringList ListerBase::supportedDevices(uint i) const
{
  if ( i!=nbGroups() ) return _groups[i]->supportedDevices();
  return supportedDevices();
}
