/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef GROUP_H
#define GROUP_H

#include <qstringlist.h>
#include <qmap.h>
#include <qvaluevector.h>
namespace Device { class Data; }

//-----------------------------------------------------------------------------
class GroupUI;

class Group
{
public:
  Group() : _gui(0) {}
  virtual ~Group() {}
  virtual void init() { initSupported(); }
  virtual QString name() const = 0;
  virtual QString label() const = 0;
  bool isSupported(const QString &device) const { return _devices.contains(device); }
  const Device::Data *deviceData(const QString &device) const;
  QStringList supportedDevices() const;
  void reinitSupported();
  GroupUI *gui() const { return _gui; }

protected:
  virtual void addDevice(const Device::Data *data);
  virtual void initSupported() = 0;

private:
  GroupUI *_gui;
  QMap<QString, const Device::Data *> _devices;

  friend class GroupUI;
};

class GroupUI
{
public:
  GroupUI(Group &group) : _group(group) {
    group._gui = this;
  }
  const Group &group() const { return _group; }

private:
  const Group &_group;
};

class ListerBase
{
public:
  virtual ~ListerBase() {}
  uint nbGroups() const { return _groups.count(); }
  const Group &group(uint i) const { return *_groups[i]; }
  const Group *group(const QString &name) const;
  bool isSupported(uint i, const QString &device) const;
  QStringList supportedDevices(uint i) const;
  QStringList supportedDevices() const;
  bool isSupported(const QString &device) const;

protected:
  void addGroup(Group *group);

private:
  QValueVector<Group *> _groups;
};

template <class GroupType>
class Lister : public ListerBase
{
public:
  const GroupType &group(uint i) const { return static_cast<const GroupType &>(ListerBase::group(i)); }
  const GroupType *group(const QString &name) const { return static_cast<const GroupType *>(ListerBase::group(name)); }
};

#endif
