/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef COMMON_GUI_H
#define COMMON_GUI_H

#include <qlayout.h>
#include <qlineedit.h>
#include <qsplitter.h>
#include <qvaluevector.h>

#include <klocale.h>
#include <kurllabel.h>
#include <kdialogbase.h>
#include <kpushbutton.h>
#define private public
#define protected public
#include <klistview.h>
#undef private
#undef protected
class KAction;

#include "common/log.h"

//-----------------------------------------------------------------------------
class BusyCursor
{
public:
  BusyCursor() { start(); }
  ~BusyCursor() { stop(); }
  static void start();
  static void stop();
  static void pause();
  static void restore();

private:
  static bool _overridePaused;
};

//-----------------------------------------------------------------------------
class Container : public QFrame
{
Q_OBJECT
public:
  enum Type { Flat, Sunken };
  Container(QWidget *parent, Type type = Flat, const char *name = 0);
  Container(QWidgetStack *stack, uint index, Type type = Flat,
            const char *name = 0);
  Container(QTabWidget *tabw, const QString &title, Type type = Flat,
            const char *name = 0);

  void initLayout();
  void addWidget(QWidget *widget, int row, int col, int alignment = 0);
  void addMultiCellWidget(QWidget *widget, int rowMin, int rowMax,
                          int colMin, int colMax, int alignment = 0);
  QGridLayout *gridLayout() { return _gridLayout; }
  uint numRows() const { return _gridLayout->numRows(); }
  uint numCols() const { return _gridLayout->numCols(); }
  void setFrame(Type type);

private:
  Type         _type;
  QGridLayout *_topLayout, *_gridLayout;
};

//-----------------------------------------------------------------------------
namespace MessageBox
{
  extern void information(const QString &text, Log::ShowMode show);
  extern void sorry(const QString &text, Log::ShowMode show);
  extern void detailedSorry(const QString &text, const QString &details, Log::ShowMode show);
  extern bool askContinue(const QString &text, const QString &caption = i18n("Warning"),
                          const KGuiItem &buttonContinue = KStdGuiItem::cont());
};

//-----------------------------------------------------------------------------
class EditListViewItem;

class ListView : public KListView
{
Q_OBJECT
public:
  ListView(QWidget *parent = 0, const char *name = 0);
  virtual void clear();
  void stopRenaming();

protected:
  virtual bool eventFilter(QObject *o, QEvent *e);

private:
  QValueList<EditListViewItem *> _editItems;

  friend class EditListViewItem;
};

class EditListViewItem : public KListViewItem
{
public:
  EditListViewItem(ListView *list);
  EditListViewItem(KListViewItem *item);
  virtual ~EditListViewItem();
  bool isRenaming() const { return _renaming; }

protected:
  virtual QWidget *editWidgetFactory(int col) const = 0;
  virtual bool alwaysAcceptEdit(int col) const = 0;
  virtual int width(const QFontMetrics &fm, const QListView *lv, int c) const;
  virtual void editDone(int col, const QWidget *editWidget);
  virtual void paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align);

private:
  bool _renaming;
  QValueVector<QWidget *> _editWidgets;

  virtual void activate() { startRename(); }
  virtual void startRename(int) { startRename(); }
  virtual void okRename(int) { renameDone(true); }
  virtual void cancelRename(int) { renameDone(false); }
  void startRename();
  void renameDone(bool force);
  void removeEditBox();

  friend class ListView;
};

//----------------------------------------------------------------------------
class PopupButton : public KPushButton
{
Q_OBJECT
public:
  PopupButton(QWidget *parent = 0, const char *name = 0);
  PopupButton(const QString &text, QWidget *parent = 0, const char *name = 0);
  void appendAction(KAction *action);
  void appendSeparator() { _separator = true; }

private:
  bool _separator;

  void init();
};

#endif
