/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef MISC_H
#define MISC_H

#include <qstringlist.h>
#include <qmap.h>
#include <qcstring.h>
#include <qvaluevector.h>
class QFont;
#undef major // to compile with Ubuntu 5.04 64-bits...
#undef minor

//-----------------------------------------------------------------------------
extern QCString repeat(const char *s, uint nb);
extern uint nbBits(uint value);
extern uint nbChars(uint value);
extern uint nbBytes(uint value);
extern QCString toHex(uint value, uint nbChars);
extern QString toHexLabel(uint value, uint nbChars);
extern QString toHexLabels(const QMemArray<uint> &values, uint nbChars);
extern uint fromHex(char c, bool *ok = 0);
extern uint fromHex(const QString &s, bool *ok = 0);
extern uint fromHexLabel(const QString &s, bool *ok);
extern uint fromHexLabel(const QString &s, uint nbChars, bool *ok);
extern uchar bin2bcd(uchar bin);
extern uchar bcd2bin(uchar bcd);
extern bool XOR(bool b1, bool b2);

extern QString prettyCString(const QCString &s);
extern QString escapeXml(const QString &s);
extern QString htmlTableRow(const QString &title, const QString &value);

extern int maxCharWidth(const QFont &font, const QString &s);
extern int maxHexCharWidth(const QFont &font);

//-----------------------------------------------------------------------------
class Range
{
public:
  Range() : start(0), end(0) {}
  Range(uint s, uint e) : start(s), end(e) {}
  bool isEmpty() const { return end<=start; }
  bool contains(uint v) const { return !isEmpty() && v>=start && v<=end; }

  uint start, end;
};

class RangeVector : public QValueVector<Range>
{
public:
  RangeVector() {}
  bool isEmpty() const {
    for (uint i=0; i<count(); i++) if ( !at(i).isEmpty() ) return false;
    return true;
  }
  bool contains(uint v) const {
    for (uint i=0; i<count(); i++) if ( at(i).contains(v) ) return true;
    return false;
  }
};

//-----------------------------------------------------------------------------
class VersionData
{
public:
  static VersionData fromString(const QString &s);
  static VersionData fromHexString(const QString &s);

public:
  VersionData(uchar major = 0, uchar mid = 0, uchar minor = 0)
    : _major(major), _mid(mid), _minor(minor) {}
  bool isValid() const { return _major!=0 || _mid!=0 || _minor!=0; }
  void clear() { _major = 0; _mid = 0; _minor = 0; }
  uchar major() const { return _major; }
  uchar mid() const { return _mid; }
  uchar minor() const { return _minor; }
  QString sub() const { return _sub; }
  QString pretty() const;
  uint toUInt() const { return (_major << 16) | (_minor << 8) | _minor; }

private:
  uchar   _major, _mid, _minor;
  QString _sub;
};

#endif
