/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2004 Alain Gibaud <alain.gibaud@free.fr>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef CONFIG_CENTER_H
#define CONFIG_CENTER_H

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qwidgetstack.h>
#include <klocale.h>

#include "tools/config_widget.h"
namespace DeviceChooser { class Button; }
class SelectDirectoryWidget;

//----------------------------------------------------------------------------
class GeneralConfigWidget : public GridConfigWidget
{
  Q_OBJECT
public:
  GeneralConfigWidget();
  virtual void loadConfig();

  virtual QString title() const { return i18n("General"); }
  virtual QString header() const { return i18n("General Configuration"); }
  virtual QPixmap pixmap() const;

public slots:
  virtual void saveConfig();

private:
  QCheckBox *_programAfterBuild, *_userIdSetToChecksum;
  QComboBox *_showDebug;
};

//----------------------------------------------------------------------------
class StandaloneConfigWidget : public ConfigWidget
{
  Q_OBJECT
public:
  StandaloneConfigWidget();
  virtual void loadConfig();
  virtual QString title() const { return i18n("Standalone File"); }
  virtual QString header() const { return i18n("Standalone File Configuration"); }
  virtual QPixmap pixmap() const;

public slots:
  virtual void saveConfig();

private:
  DeviceChooser::Button *_device;
  ToolsConfigWidget     *_tools;
};

//----------------------------------------------------------------------------
class ToolchainConfigWidget : public GridConfigWidget
{
  Q_OBJECT
public:
  ToolchainConfigWidget(const Tool::Group &group, QWidget *parent);
  virtual void loadConfig();

public slots:
  virtual void saveConfig();

private slots:
  void changed();

private:
  const Tool::Group     &_group;
  QCheckBox             *_withWine;
  SelectDirectoryWidget *_dir;
  QLabel                *_founds[Tool::Nb_Categories];
};

//----------------------------------------------------------------------------
class ToolchainsConfigWidget : public GridConfigWidget
{
  Q_OBJECT
public:
  ToolchainsConfigWidget();
  virtual void loadConfig();
  virtual QString title() const { return i18n("Toolchains"); }
  virtual QString header() const { return i18n("Toolchains Configuration"); }
  virtual QPixmap pixmap() const;

public slots:
  virtual void saveConfig();

private slots:
  void changed(int i);

private:
  QComboBox    *_combo;
  QWidgetStack *_stack;
};

//----------------------------------------------------------------------------
class ConfigCenter : public Dialog
{
Q_OBJECT
public:
  enum Type { General = 0, ProgSelect, ProgOptions, Tools, Standalone, Nb_Types };
  ConfigCenter(Type showType, QWidget *parent);

public slots:
  virtual void slotOk();
  virtual void slotApply();

private:
  ConfigWidget *_configWidgets[Nb_Types];

  static ConfigWidget *factory(Type type);
};

#endif
