/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef DEBUG_MANAGER_H
#define DEBUG_MANAGER_H

#include <qvaluelist.h>
#include <qvaluevector.h>
#include <qtimer.h>

#include "common/log.h"

class PURL;
class Editor;
class KBEditor;
namespace Coff { class SectionParser; }
namespace Register { class View; }
namespace Device { class Data; }

class DebugManager : public QObject, public Log::Manager
{
Q_OBJECT
public:
  DebugManager(QObject *parent);
  virtual ~DebugManager();
  void addEditor(Editor *editor);
  void clearEditors();
  void init();
  void clear();

  void run();
  void halt();
  void reset();
  void step();

  void readAll();
  void stopWatchAll();

public slots:
  bool update();
  void updateAsmEditors();
  bool updateRegisterViews();
  void setRegisterWatched(uint index, bool watched);
  bool readRegister(uint index);
  bool writeRegister(uint index, uint value);

signals:
  void statusChanged(const QString &text);
  void targetStateChanged();
  void actionMessage(const QString &text);

private slots:
  void editorDestroyed();
  void slotTimeout();
  void updateDevice();

private:
  const Device::Data  *_data;
  Coff::SectionParser *_coffParser;
  QValueList<Editor *> _editors;
  uint _pc;
  QValueVector<uint> _breakpoints;
  QTimer             _timer;
  QValueList<uint> _watchedRegisters;

  bool updateEditorPC(KBEditor *editor) const;
  QValueVector<uint> computeBreakpoints();
  Register::View *registerView() const;
  bool isActive() const;
};

#endif
