/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "generic_device.h"

#include <klocale.h>

#include "device_group.h"
#include "gui/device_group_ui.h"

//-----------------------------------------------------------------------------
const Device::StatusData Device::STATUS_DATA[Nb_Status] = {
  { "IP",     I18N_NOOP("In Production")                  },
  { "Future", I18N_NOOP("Future Product")                 },
  { "NR",     I18N_NOOP("Not Recommended for New Design") },
  { "EOL",    I18N_NOOP("End Of Life"  )                  },
  { "?",      I18N_NOOP("Unknown")                        }
};

const uint Device::INVALID = 0xFFFFFFFFU;

const Device::FrequencyRange::Data Device::FrequencyRange::TYPE_DATA[Nb_Types] = {
  { "commercial", I18N_NOOP("Commercial") },
  { "industrial", I18N_NOOP("Industrial") },
  { "extended",   I18N_NOOP("Extended")   }
};

const Device::FrequencyRange::Data Device::FrequencyRange::SPECIAL_DATA[Nb_Specials] = {
  { "",            I18N_NOOP("Normal")      },
  { "low power",   I18N_NOOP("Low Power")   },
  { "low voltage", I18N_NOOP("Low Voltage") }
};

//-----------------------------------------------------------------------------
Device::Array::Array(const Array &array)
  : _data(array.size())
{
  for (uint i=0; i<array.size(); i++) _data[i] = array[i];
}

Device::Array &Device::Array::operator +=(const Array &a)
{
  uint s = _data.size();
  _data.resize(s + a.size());
  for (uint i=0; i<a.size(); i++) _data[s+i] = a[i];
  return *this;
}

Device::Array Device::Array::mid(uint index, int s) const
{
  Q_ASSERT( index<size() && s<=int(size()-index) && s>=-1);
  Array array(s==-1 ? size()-index : s);
  for (uint i=0; i<array.size(); i++) array[i] = _data[index+i];
  return array;
}

Device::Array &Device::Array::operator =(const Array &array)
{
  _data.resize(array.size());
  for (uint i=0; i<array.size(); i++) _data[i] = array[i];
  return *this;
}

//-----------------------------------------------------------------------------
Device::Data::Data(const ConstData &cdata)
  : _group(0)
{
  _name = cdata.name;
  _document = cdata.document;
  _status = cdata.status;
  _alternatives = QStringList::split(' ', cdata.alternatives);
  _architecture = cdata.architecture;
  for (uint iw=0; cdata.frequencyRanges[iw].type!=FrequencyRange::Nb_Types; iw++) {
    const ConstFrequencyRangeData cf = cdata.frequencyRanges[iw];
    FrequencyRange fr;
    fr.type = cf.type;
    fr.special = cf.special;
    for (uint ir=0; cf.vdds[ir].start.x!=-1; ir++)
      fr.vdds.append(cf.vdds[ir]);
    _frequencyRanges.append(fr);
  }
}

const Device::GroupUI &Device::Data::groupui() const
{
  return static_cast<const GroupUI &>(*_group->gui());
}
