/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef GENERIC_DEVICE_H
#define GENERIC_DEVICE_H

#include <qstringlist.h>

#include "common/misc.h"

namespace Device
{
//----------------------------------------------------------------------------
enum Status { Status_IP = 0, Status_Future, Status_NR, Status_EOL,
              Status_Unknown, Nb_Status };
struct StatusData {
  const char *name, *label;
};
extern const StatusData STATUS_DATA[Nb_Status];

extern const uint INVALID; // used to mark a word as "non initialized"

struct RangeBox {
  struct Value { double x, yMin, yMax; };
  Value start, end;
};
class FrequencyRange {
public:
  enum Type { Commercial = 0, Industrial, Extended, Nb_Types };
  enum Special { Normal = 0, LowPower, LowVoltage, Nb_Specials };
  Type type;
  Special special;
  QValueVector<RangeBox> vdds;
  struct Data {
    const char *name, *label;
  };
  static const Data TYPE_DATA[Nb_Types];
  static const Data SPECIAL_DATA[Nb_Specials];
};

struct IdData {
  uint revision, variant, minorRevision, process;
};
typedef QMap<QString, IdData> IdList;

//----------------------------------------------------------------------------
class XmlToDataBase;
class Group;
class GroupUI;

struct ConstFrequencyRangeData {
  FrequencyRange::Type type;
  FrequencyRange::Special special;
  RangeBox vdds[5];
};

struct ConstData {
  const char *name, *document, *alternatives;
  Status status;
  uint architecture;
  ConstFrequencyRangeData frequencyRanges[5];
};

//----------------------------------------------------------------------------
// we don't want implicit sharing
class Array
{
public:
  Array(uint size = 0) : _data(size) {}
  Array(const Array &array);
  Array &operator =(const Array &);
  Array &operator +=(const Array &a);
  Array mid(uint index, int size = -1) const;
  void resize(uint s) { _data.resize(s); }
  uint size() const { return _data.size(); }
  uint count() const { return _data.size(); }
  uint operator [](uint i) const { return _data[i]; }
  uint &operator [](uint i) { return _data[i]; }
  bool operator ==(const Array &array) const { return _data==array._data; }

private:
  QMemArray<uint> _data;
};

//----------------------------------------------------------------------------
class Data
{
public:
  Data() : _group(0) {}
  Data(const ConstData &cdata);
  virtual ~Data() {}
  const Group &group() const { return *_group; }
  const GroupUI &groupui() const;
  QString name() const { return _name; }
  virtual QString fname(Device::FrequencyRange::Special) const { return _name; }
  Status status() const { return _status; }
  QString document() const { return _document; }
  QStringList alternatives() const { return _alternatives; }
  virtual bool matchId(uint rawId, uint deviceIdAddress, IdData &idata) const = 0;
  const QValueVector<FrequencyRange> &frequencyRanges() const { return _frequencyRanges; }
  virtual uint nbBitsAddress() const = 0;
  uint nbBytesAddress() const { return nbBitsAddress()/8 + (nbBitsAddress()%8 ? 1 : 0); }
  uint nbCharsAddress() const { return nbBitsAddress()/4 + (nbBitsAddress()%4 ? 1 : 0); }
  virtual bool canWriteCalibration() const = 0; // #### REMOVE ME
  virtual uint nbRegisters() const = 0;

protected:
  const Group *_group;
  QString      _name, _document;
  QStringList  _alternatives;
  Status       _status;
  uint         _architecture;
  QValueVector<FrequencyRange> _frequencyRanges;

  friend class XmlToDataBase;
  friend class Group;
};

}; // namespace

#endif
