/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef REGISTER_VIEW_H
#define REGISTER_VIEW_H

#include <qframe.h>

#include "common/gui/misc_gui.h"

namespace Device
{
class Data;

//-----------------------------------------------------------------------------
class RegisterItem : public EditListViewItem
{
public:
  RegisterItem(const Device::Data &data, uint index, KListViewItem *item);
  void init();
  uint index() const { return _index; }
  void setValue(uint value);

protected:
  const Device::Data &_data;
  uint _index, _value;
  bool _changed;

  virtual void paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align);
  virtual QWidget *editWidgetFactory(int col) const;
  virtual bool alwaysAcceptEdit(int) const { return true; }
  virtual void editDone(int col, const QWidget *editWidget);
  virtual uint address() const = 0;
  virtual uint nbCharsAddress() const = 0;
  virtual uint nbCharsValue() const = 0;
  virtual QString name() const = 0;
};

//-----------------------------------------------------------------------------
class RegisterView : public QFrame
{
Q_OBJECT
public:
  RegisterView(const Device::Data &data, bool debugging, QWidget *parent, const char *name)
    : QFrame(parent, name), _data(data) { Q_UNUSED(debugging); }
  virtual void setActive(bool active) = 0;
  virtual void setWatched(uint index, bool watched) = 0;
  virtual void setRegisterValue(uint index, uint value) = 0;

signals:
  void readSignal(uint i);
  void writeSignal(uint i, uint value);
  void setWatchedSignal(uint i, bool watched);

protected:
  const Device::Data &_data;
};

}; // namespace

#endif
