/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "mem24_prog.h"

#include <klocale.h>

#include "common/gui/misc_gui.h"
#include "devices/base/device_list.h"
#include "progs/base/prog_config.h"
#include "progs/base/prog_group.h"

//-----------------------------------------------------------------------------
Device::Array Programmer::Mem24DeviceSpecific::read()
{
  setPowerOn();
  Device::Array array = doRead();
  setPowerOff();
  return array;
}

bool Programmer::Mem24DeviceSpecific::write(const Device::Array &data)
{
  setPowerOn();
  doWrite(data);
  setPowerOff();
  return !hasError();
}

bool Programmer::Mem24DeviceSpecific::verifyByte(uint index, uint d, const Mem24::Memory &memory,
                                                 Programmer::VerifyActions actions)
{
  uint v = memory.byte(index);
  if ( v==d ) return true;
  uint address = index;
  if ( actions & Programmer::BlankCheckVerify )
    log(Log::Error, i18n("Device memory is not blank (at address 0x%2: reading 0x%3 and expecting 0x%4).")
                    .arg(toHex(address, device().nbCharsAddress())).arg(toHex(d, 2)).arg(toHex(v, 2)));
  else log(Log::Error, i18n("Device memory doesn't match hex file (at address 0x%2: reading 0x%3 and expecting 0x%4).")
                       .arg(toHex(address, device().nbCharsAddress())).arg(toHex(d, 2)).arg(toHex(v, 2)));
  return false;
}

bool Programmer::Mem24DeviceSpecific::verify(const Mem24::Memory &memory, Programmer::VerifyActions actions)
{
  setPowerOn();
  bool ok = doVerify(memory, actions);
  setPowerOff();
  return ok;
}

bool Programmer::Mem24DeviceSpecific::doVerify(const Mem24::Memory &memory, Programmer::VerifyActions actions)
{
  Device::Array data = doRead();
  if ( hasError() ) return false;
  for (uint i=0; i<data.count(); i++)
    if ( !verifyByte(i, data[i], memory, actions) ) return false;
  return true;
}

//----------------------------------------------------------------------------
uint Programmer::Mem24Base::nbSteps(Operation operation) const
{
  uint length = Base::nbSteps(operation);
  if ( operation==Connect ) return length;
  if ( operation==Write ) length += device()->nbBytes();
  length += device()->nbBytes();
  return length;
}

//----------------------------------------------------------------------------
bool Programmer::Mem24Base::internalErase()
{
  initProgramming();
  addProgress(OPERATION_DATA[Erase].nbSteps);
  Mem24::Memory memory(*device());
  return specific()->write(memory.arrayForWriting());
}

bool Programmer::Mem24Base::internalRead(Device::Memory &memory)
{
  initProgramming();
  Mem24::Memory &pmemory = static_cast<Mem24::Memory &>(memory);
  Device::Array data = specific()->read();
  if ( hasError() ) return false;
  for (uint i=0; i<data.count(); i++) pmemory.setByte(i, data[i]);
  return true;
}

bool Programmer::Mem24Base::internalProgram(const Device::Memory &memory)
{
  initProgramming();
  addProgress(OPERATION_DATA[Erase].nbSteps);
  const Mem24::Memory &pmemory = static_cast<const Mem24::Memory &>(memory);
  const Device::Array &data = pmemory.arrayForWriting();
  if ( !specific()->write(data) ) return false;
  VerifyActions actions = IgnoreProtectedVerify;
  if ( Config::onlyVerifyProgrammed() ) actions |= OnlyProgrammedVerify;
  return specific()->verify(pmemory, actions);
}

bool Programmer::Mem24Base::internalVerify(const Device::Memory &memory, VerifyActions actions)
{
  initProgramming();
  const Mem24::Memory &pmemory = static_cast<const Mem24::Memory &>(memory);
  return specific()->verify(pmemory, actions);
}
