/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "pic_config_editor.h"

#include <qlayout.h>
#include <qvgroupbox.h>
#include <qapplication.h>

#include "devices/pic/pic_memory.h"
#include "pic_config_word_editor.h"
#include "common/misc.h"

//----------------------------------------------------------------------------
TabWidget::TabWidget(QWidget *parent)
  : KTabWidget(parent)
{
  setTabBar(new TabBar(this, "tabbar"));
}

void TabWidget::wheelEvent(QWheelEvent *e)
{
  e->ignore();
}

void TabBar::wheelEvent(QWheelEvent *e)
{
  QApplication::sendEvent(parent(), e);
}

//----------------------------------------------------------------------------
Pic::MemoryConfigEditorWidget::MemoryConfigEditorWidget(Memory &memory, QWidget *parent)
  : Device::MemoryEditorGroup(&memory, parent, "pic_config_editor_widget"),
    MemoryCaster(MemoryConfig, memory)
{
  QHBoxLayout *hb = new QHBoxLayout(_top);

  TabWidget *tabw = 0;
  uint nbWords = device().nbWords(MemoryConfig);
  if ( nbWords>1 ) {
    tabw = new TabWidget(this);
    hb->addWidget(tabw);
  }

  for(uint i=0; i<nbWords; ++i) {
    //qDebug("BinWordsEditor for config word #%i", i);
    //uint address = device().range(Device::MemoryConfig).start + device().addressIncrement(Device::MemoryConfig) * i;
    //qDebug("address: %s %s nb: %i", toHex(address, 8).data(), device().configWord(i).name.latin1(), device().configWord(i).masks.count());
    if ( device().config()._words[i].masks.count()==0 ) continue;
    QWidget *page = 0;
    if ( nbWords>1 ) {
      page = new QWidget(tabw);
      tabw->addTab(page, device().config()._words[i].name);
    } else {
      page = new QGroupBox(this);
      hb->addWidget(page);
    }
    QVBoxLayout *vbox = new QVBoxLayout(page, 10, 10);
    QHBoxLayout *hbox = new QHBoxLayout(vbox);
    ConfigWordEditor *we = new ConfigWordEditor(memory, i, page);
    addEditor(we);
    hbox->addWidget(we);
    hbox->addStretch(1);
    vbox->addStretch(1);
  }
}

//----------------------------------------------------------------------------
Pic::MemoryConfigEditor::MemoryConfigEditor(const HexView &hexview, Memory &memory, QWidget *parent)
  : MemoryTypeEditor(hexview, MemoryConfig, memory, parent, "pic_config_editor")
{}

void Pic::MemoryConfigEditor::init()
{
  MemoryTypeEditor::init();
  MemoryConfigEditorWidget *w = new MemoryConfigEditorWidget(memory(), this);
  addEditor(w);
  _top->addWidget(w);
}
