/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "pic_config_word_editor.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qcombobox.h>
#include <klocale.h>

#include "common/misc.h"

Pic::ConfigWordEditor::ConfigWordEditor(Memory &memory, uint ci, QWidget *parent)
  : MemoryEditor(MemoryConfig, memory, parent, "pic_config_word_editor"), _configIndex(ci)
{
  QHBoxLayout *hbox = new QHBoxLayout(_top);
  _mdb = new MemoryRangeEditor(MemoryConfig, memory, 1, 1, ci, 1, this);
  _mdb->init();
  connect(_mdb, SIGNAL(modified()), SIGNAL(modified()));
  connect(_mdb, SIGNAL(modified()), SLOT(updateDisplay()));
  hbox->addWidget(_mdb);
  QLabel *ulabel = new QLabel(this);
  hbox->addWidget(ulabel);
  hbox->addStretch(1);

  QGridLayout *grid = new QGridLayout(_top);
  grid->setColStretch(2, 1);
  const Config::Word &cword = device().config()._words[ci];
  _combos.resize(cword.masks.count());
  uint nbChars = device().nbCharsWord(MemoryConfig);
  for (uint k=0; k<_combos.count(); k++) {
    const Config::Mask &cmask = cword.masks[k];
    QLabel *label = new QLabel(Config::maskLabel(cmask.name) + ":", this);
    grid->addWidget(label, k, 0);
    label = new QLabel(cmask.name, this);
    grid->addWidget(label, k, 1);
    _combos[k] = new ComboBox(this);
    for (uint i=0; i<cmask.values.count(); i++) {
      QString label = Config::valueLabel(cmask.name, cmask.values[i].name);
      label += " (" + toHexLabel(cmask.values[i].value, nbChars) + ")";
      _combos[k]->insertItem(label);
    }
    connect(_combos[k], SIGNAL(activated(int)), SLOT(slotModified()));
    grid->addWidget(_combos[k], k, 2);
  }

  ulabel->setText(i18n("(Read mask: %1 - Write mask: %2)")
    .arg(toHexLabel(cword.rmask, nbChars)).arg(toHexLabel(cword.wmask, nbChars)));
}

void Pic::ConfigWordEditor::setReadOnly(bool readOnly)
{
  _mdb->setReadOnly(readOnly);
  const Config::Word &cword = device().config()._words[_configIndex];
  for (uint k=0; k<_combos.count(); k++) {
    const Config::Mask &cmask = cword.masks[k];
    _combos[k]->setEnabled(!readOnly && !(cmask.value & cword.pmask) && cmask.values.count()!=1);
  }
}

void Pic::ConfigWordEditor::slotModified()
{
  uint v = memory().normalizedWord(MemoryConfig, _configIndex);
  //qDebug("BinWordEditor::slotModified %i: %s", _configIndex, toHex(v, 4).data());
  for (uint k=0; k<_combos.count(); k++) {
    const Config::Mask &cmask = device().config()._words[_configIndex].masks[k];
    v &= ~cmask.value; // clear masked bits
    v |= cmask.values[_combos[k]->currentItem()].value; // set value
  }
  memory().setWord(MemoryConfig, _configIndex, v);
  //qDebug("  now: %s", toHex(v, 4).data());
  _mdb->updateDisplay();
  emit modified();
}

void Pic::ConfigWordEditor::updateDisplay()
{
  uint v = memory().normalizedWord(MemoryConfig, _configIndex);
  //qDebug("BinWordEditor::updateDisplay %i: %s", _configIndex, toHex(v, 4).data());
  for (uint k=0; k<_combos.count(); k++) {
    const Config::Mask &cmask = device().config()._words[_configIndex].masks[k];
    for (int i=cmask.values.count()-1; i>=0; i--) {
      if ( (v & cmask.values[i].value)==cmask.values[i].value ) {
        _combos[k]->setCurrentItem(i);
        break;
      }
    }
  }
  _mdb->updateDisplay();
}
