/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PIC_REGISTER_VIEW_H
#define PIC_REGISTER_VIEW_H

#include <qvaluevector.h>
class QPushButton;
class QCheckBox;

#include "devices/base/gui/register_view.h"
#include "devices/pic/pic.h"
namespace Device { class RegisterHexWordEditor; };

namespace Pic
{
//-----------------------------------------------------------------------------
class BankWidget : public QFrame
{
Q_OBJECT
public:
  BankWidget(uint bank, const Data &data, bool debugging, QWidget *parent);
  void setActive(bool active);
  void readAll();
  void stopWatchAll();
  void setWatched(uint i, bool watched);
  void setRegisterValue(uint i, uint value);

signals:
  void readSignal(uint i);
  void writeSignal(uint i, uint value);
  void setWatchedSignal(uint i, bool watched);

private slots:
  void read();
  void write();
  void toggleWatch();

private:
  const Data &_data;
  uint _index;
  QValueVector<QPushButton *> _buttons;
  QValueVector<QCheckBox *> _watch;
  QValueVector<Device::RegisterHexWordEditor *> _words;

  uint registerIndex(uint i) const;
  uint registerAddress(uint i) const;
  void read(uint i) { emit readSignal(registerIndex(i)); }
};

//-----------------------------------------------------------------------------
class RegisterView : public Device::RegisterView
{
Q_OBJECT
public:
  RegisterView(const Data &data, bool debugging, QWidget *parent);
  virtual void setActive(bool active);
  virtual void setWatched(uint index, bool watched);
  virtual void setRegisterValue(uint index, uint value);

public slots:
  virtual void readAll();
  virtual void stopWatchAll();

private:
  QPushButton *_readAllButton, *_clearAllButton;
  QValueVector<BankWidget *> _banks;
};

//-----------------------------------------------------------------------------
class RegisterItem : public Device::RegisterItem
{
public:
  RegisterItem(const Device::Data &data, uint index, KListViewItem *item)
    : Device::RegisterItem(data, index, item) {}
  const Data &data() const { return static_cast<const Data &>(_data); }
  virtual uint address() const { return data().nbBytesRegister() * _index; }
  virtual uint nbCharsAddress() const { return data().nbCharsRegisterAddress(); }
  virtual uint nbCharsValue() const { return data().nbCharsRegister(); }
  virtual QString name() const { return data().registerName(address()); }
};

}; // namespace

#endif
