/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "pic_group.h"

#include "device_data.h"
#include "pic_memory.h"

void Pic::Group::initSupported()
{
  for (uint i=0; CDATA_LIST[i]; i++) {
    const ConstData &cdata = *CDATA_LIST[i];
    Data *data = new Data(cdata.gdata);

    data->_id = cdata.id;
    data->_memoryTechnology = cdata.memoryTechnology;
    for (uint i=0; i<Nb_ProgVoltageTypes; i++) {
      data->_voltages[i].min = cdata.voltages[i].min;
      data->_voltages[i].max = cdata.voltages[i].max;
      data->_voltages[i].nominal = cdata.voltages[i].nominal;
    }
    for (uint i=0; i<Nb_MemoryRangeTypes; i++)
      data->_ranges[i] = cdata.ranges[i];
    for (uint iw=0; cdata.configs[iw].name; iw++) {
      const ConstConfigWord &ccword = cdata.configs[iw];
      Config::Word cword;
      cword.name = ccword.name;
      cword.rmask = ccword.rmask;
      cword.wmask = ccword.wmask;
      cword.pmask = ccword.pmask;
      cword.cmask = ccword.cmask;
      cword.bvalue = ccword.bvalue;
      for (uint im=0; ccword.masks[im].name; im++) {
        const ConstConfigMask &ccmask = ccword.masks[im];
        Config::Mask cmask;
        cmask.name = ccmask.name;
        for (uint k=0; ccmask.icnames[k]; k++)
          cmask.ignoredCNames += ccmask.icnames[k];
        cmask.value = ccmask.value;
        for (uint iv=0; ccmask.values[iv].name; iv++) {
          Config::Value cvalue;
          cvalue.name = ccmask.values[iv].name;
          cvalue.cname = ccmask.values[iv].cname;
          for (uint k=0; ccmask.values[iv].ecnames[k]; k++)
            cvalue.extraCNames += ccmask.values[iv].ecnames[k];
          cvalue.value = ccmask.values[iv].value;
          cmask.values.append(cvalue);
        }
        cword.masks.append(cmask);
      }
      data->_config._words.append(cword);
    }
    data->_calibration = cdata.calibration;
     data->_nbBanks = cdata.nbBanks;
    for (uint i=0; cdata.sfrs[i].name; i++)
      data->_sfrs[cdata.sfrs[i].name] = cdata.sfrs[i].address;
    for (uint i=0; ; i++) {
      QValueVector<RangeData> m;
      for (uint k=0; cdata.mirrored[i].ranges[k].length!=0; k++)
        m.append(cdata.mirrored[i].ranges[k]);
      if ( m.isEmpty() ) break;
      data->_mirrored.append(m);
    }
    for (uint i=0; cdata.unused[i].length!=0; i++)
      data->_unused.append(cdata.unused[i]);

    addDevice(data);
  }
}

Device::Memory *Pic::Group::createMemory(const Device::Data &data) const
{
  return new Memory(static_cast<const Pic::Data &>(data));
}
