/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef EDITOR_MANAGER_H
#define EDITOR_MANAGER_H

class QEvent;
#include <qtabwidget.h>
#include <qtabbar.h>

#include "text_editor.h"

//-----------------------------------------------------------------------------
class EditorTabBar : public QTabBar
{
Q_OBJECT
public:
  EditorTabBar(QWidget *parent) : QTabBar(parent, "editor_tab_bar") {}
  void setReadOnly(uint index, bool readOnly) { _readOnly[tabAt(index)] = readOnly; }

private:
  QMap<QTab *, bool> _readOnly;
  virtual void paintLabel(QPainter *p, const QRect &br, QTab *t, bool has_focus) const;
};

//-----------------------------------------------------------------------------
class EditorManager : public QTabWidget
{
  Q_OBJECT
public:
  EditorManager(QWidget *parent);

  PURL::List files() const;
  QValueList<Editor *> &editors() { return _editors; }
  uint nbEditors() const { return _editors.count(); }
  Editor *createEditor(PURL::FileType type, const PURL &url);
  void addEditor(Editor *e);
  Editor *currentEditor() const { return _current; }
  Editor *findEditor(const PURL &file);
  Editor *findEditor(const QString &name);
  void showEditor(Editor *e);
  bool closeEditor(const PURL &url);
  bool closeEditor(Editor *e, bool ask);
  bool openFile(const PURL &url);
  Editor *openEditor(const PURL &url);

  void clearMarks(KBEditor::MarkTypes types);

public slots:
  void updateDisplay();
  void slotDropEvent(QDropEvent *e);
  void saveAllFiles();
  bool closeCurrentEditor();
  bool closeAllEditors();
  bool closeAllOtherEditors();

signals:
  void modified(const PURL &url);
  void guiChanged();
  void statusChanged(const QString &);

private:
  void changeToEditor(Editor *e);
  void connectEditor(Editor *editor);
  void disconnectEditor(Editor *editor);
  void enableActions(bool enable);
  QString title(const Editor *e) const;
  virtual bool eventFilter(QObject *watched, QEvent *e);

private slots:
  void showEditor(QWidget *w) { showEditor(static_cast<Editor *>(w)); }
  void modifiedSlot();

private:
  Editor *_current;
  QValueList<Editor *> _editors;
};

#endif
