/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "global_config.h"

#include <kapplication.h>
#include <kconfig.h>
#include <klocale.h>

#include "devices/base/device_list.h"
#include "progs/base/prog_list.h"

//-----------------------------------------------------------------------------
KConfig &GlobalConfig::config()
{
  KConfig *conf = kapp->config();
  conf->setGroup(QString::null);
  return *conf;
}

PURL GlobalConfig::openedProject()
{
  return PURL(config().readEntry("project", QString::null));
}
void GlobalConfig::writeOpenedProject(const PURL &p)
{
  config().writeEntry("project", p.filepath());
}

PURL::List GlobalConfig::openedFiles()
{
  PURL::List files;
  uint i = 0;
  for (;;) {
    QString file = config().readEntry(QString("file%1").arg(i), QString::null);
    if ( file.isEmpty() ) break;
    files += PURL(file);
    i++;
  }
  return files;
}
void GlobalConfig::writeOpenedFiles(const PURL::List &files)
{
  for (uint i=0; i<=files.count(); i++) {
    QString s = (i==files.count() ? QString::null : files[i].filepath());
    config().writeEntry(QString("file%1").arg(i), s);
  }
}

void GlobalConfig::writeProgrammerGroup(const Programmer::Group &group)
{
  config().writeEntry("programmer", group.name());
}
const Programmer::Group &GlobalConfig::programmerGroup()
{
  QString s = config().readEntry("programmer");
  for (uint i=0; i<Programmer::lister().nbGroups(); i++) {
    const Programmer::Group &group = Programmer::lister().group(i);
    if ( s==group.name() ) return group;
  }
  return Programmer::lister().group(0);
}

void GlobalConfig::writeDebugLevel(Log::DebugLevel level)
{
  config().writeEntry("log_debug_level", level);
}
Log::DebugLevel GlobalConfig::debugLevel()
{
  uint level = config().readUnsignedNumEntry("log_debug_level", Log::Nb_DebugLevels);
  if ( level>Log::Nb_DebugLevels ) return Log::Nb_DebugLevels;
  return Log::DebugLevel(level);
}

void GlobalConfig::writeProgramAfterBuild(bool program)
{
  config().writeEntry("program_after_build", program);
}
bool GlobalConfig::programAfterBuild()
{
  return config().readBoolEntry("program_after_build", false);
}

void GlobalConfig::writeUserIdSetToChecksum(bool set)
{
  config().writeEntry("user_id_set_to_checksum", set);
}
bool GlobalConfig::isUserIdSetToChecksum()
{
  return config().readBoolEntry("user_id_set_to_checksum", false);
}

//-----------------------------------------------------------------------------
KConfig &DeviceChooser::Config::config()
{
  KConfig *conf = kapp->config();
  conf->setGroup("device_chooser");
  return *conf;
}

void DeviceChooser::Config::writeListType(Dialog::ListType type)
{
  config().writeEntry("list_type", type);
}
DeviceChooser::Dialog::ListType DeviceChooser::Config::listType()
{
  uint t = config().readUnsignedNumEntry("list_type", Dialog::FamilyTree);
  t = QMIN(t, Dialog::Nb_ListTypes-1);
  return Dialog::ListType(t);
}

void DeviceChooser::Config::writeProgrammerIndex(uint i)
{
  config().writeEntry("programmer_index", i);
}
uint DeviceChooser::Config::programmerIndex()
{
  return config().readUnsignedNumEntry("programmer_index", 0);
}

void DeviceChooser::Config::writeCompilerIndex(uint i)
{
  config().writeEntry("compiler_index", i);
}
uint DeviceChooser::Config::compilerIndex()
{
  return config().readUnsignedNumEntry("compiler_index", 0);
}

bool DeviceChooser::Config::filterEnabled()
{
  return config().readBoolEntry("filter_enabled", false);
}
void DeviceChooser::Config::writeFilterEnabled(bool enabled)
{
  config().writeEntry("filter_enabled", enabled);
}

//-----------------------------------------------------------------------------
KConfig &GuiConfig::config()
{
  KConfig *conf = kapp->config();
  conf->setGroup("gui");
  return *conf;
}
