/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "port.h"

#include <sys/time.h>
#include <klocale.h>

#include "serial.h"
#include "parallel.h"
#include "usb_port.h"

//-----------------------------------------------------------------------------
void Port::msleep(uint ms)
{
//  struct timespec ts;
//  ts.tv_sec = ms / 1000;              // seconds
//  ts.tv_nsec = ms * 1000000;          // nanoseconds
//  nanosleep(&ts, 0); // #### not portable ?
  usleep(ms*1000);
}

// from Brian C Lane's code
// works better than usleep
void Port::usleep(uint us)
{
  if ( us==0 ) return;
  struct timeval tv, wt;
  gettimeofday(&tv, 0);
  wt.tv_usec = (tv.tv_usec + us) % 1000000;
  wt.tv_sec = tv.tv_sec + (tv.tv_usec + us) / 1000000;

  for (;;) {
    gettimeofday(&tv, 0);
    if ( tv.tv_sec>wt.tv_sec ) return;
    if ( tv.tv_sec==wt.tv_sec && tv.tv_usec>wt.tv_usec ) return;
  }
}

//-----------------------------------------------------------------------------
const Port::Data Port::DATA[Nb_Types] = {
  { I18N_NOOP("Serial Port"),   "serial",   true  },
  { I18N_NOOP("Parallel Port"), "parallel", true  },
  { I18N_NOOP("USB Port"),      "usb",      false }
};


QStringList Port::probedDeviceList(Type type)
{
  if ( !isAvailable(type) ) return QStringList();
  switch (type) {
    case SerialType: return Serial::probedDeviceList();
    case ParallelType: return Parallel::probedDeviceList();
    case USBType: break;
    case Nb_Types: Q_ASSERT(false); break;
  }
  return QStringList();
}

bool Port::isAvailable(Type type)
{
  switch (type) {
    case SerialType: return Serial::isAvailable();
    case ParallelType: return Parallel::isAvailable();
    case USBType: return USB::isAvailable();
    case Nb_Types: Q_ASSERT(false); break;
  }
  return false;
}
