/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "port_base.h"

#include <errno.h>
#include <klocale.h>

bool Port::Base::open()
{
  close();
  resetError();
  _closing = false;
  return internalOpen();
}

void Port::Base::close()
{
  _closing = true;
  internalClose();
}

bool Port::Base::send(const char *, uint)
{
  qFatal("send not implemented");
  return false;
}
QCString Port::Base::receive(uint)
{
  qFatal("receive not implemented");
  return QCString();
}

bool Port::Base::setPinOn(uint, bool, LogicType)
{
  qFatal("setPinOn not implemented");
  return false;
}
bool Port::Base::readPin(uint, LogicType)
{
  qFatal("readPin not implemented");
  return 0;
}
QValueVector<Port::PinData> Port::Base::pinData(IOType) const
{
  qFatal("pinData not implemented");
  return QValueVector<PinData>();
}
bool Port::Base::isGroundPin(uint) const
{
  qFatal("isGroundPin not implemented");
  return false;
}
uint Port::Base::groundPin() const
{
  qFatal("groundPin not implemented");
  return 0;
}
Port::IOType Port::Base::ioType(uint) const
{
  qFatal("ioType not implemented");
  return NoIO;
}

void Port::Base::log(Log::Type logType, const QString &message)
{
  if ( logType==Log::Error && _closing ) return;
  Log::Client::log(logType, i18n(DATA[type()].label) + ": " + message);
  if ( logType==Log::Error ) close();
}

void Port::Base::log(Log::DebugLevel level, const QString &message)
{
  Log::Client::log(level, i18n(DATA[type()].label) + ": " + message);
}

void Port::Base::setLError(const QString &message)
{
  log(Log::Error, message + QString(" (errno=%1).").arg(strerror(errno)));
}

void Port::Base::logData(const QString &)
{
/*
  QString vs;
  for (uint i=0; i<s.length(); i++) {
    char c = s[i];
    switch (c) {
      case '\r': vs += "\\r"; break;
      case '\n': vs += "\\n"; break;
      case '<': vs += "&lt;"; break;
      case '>': vs += "&gt;"; break;
      default: {
        if ( c>=32 && c<=126 ) vs += c;
        else {
          QString tmp;
          tmp.sprintf("\\x%02x", c);
          vs += tmp;
        }
        break;
      }
    }
  }
  qDebug("%s", vs.latin1());
*/
  //log(Log::Debug, vs);
}
