/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PORT_BASE_H
#define PORT_BASE_H

#include <qvaluevector.h>

#include "port.h"
#include "common/log.h"

namespace Port
{

class Base : public Log::Client
{
public:
  Base(Log::Base &base) : Log::Client(base) {}
  virtual Type type() const = 0;
  virtual QString device() const = 0;
  bool open();
  void close();

  virtual void log(Log::Type type, const QString &message);
  virtual void log(Log::DebugLevel level, const QString &message);
  void logData(const QString &s);

  virtual bool send(const char *data, uint size);
  virtual QCString receive(uint size);

  virtual bool setPinOn(uint pin, bool on, LogicType type);
  virtual bool readPin(uint pin, LogicType type);
  virtual QValueVector<PinData> pinData(IOType ioType) const;
  virtual bool isGroundPin(uint pin) const;
  virtual uint groundPin() const;
  virtual IOType ioType(uint pin) const;

protected:
  virtual bool internalOpen() = 0;
  virtual void internalClose() = 0;
  void setLError(const QString &message);

private:
  bool _closing;
};

}; // namespace

#endif
