/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *   Copyright (C) 2002-2003 Stephen Landamore <stephen@landamore.com>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef SERIAL_H
#define SERIAL_H

#include <termios.h>

#include "port_base.h"

namespace Port
{

class Serial : public Base
{
public:
  enum Speed { S0 = 0, S50, S75, S110, S134, S150, S200, S300, S600, S1200,
               S1800, S2400, S4800, S9600, S19200, S38400, S57600, S115200,
               Nb_Speeds };
  enum Property { NoProperty = 0, NeedDrain = 1, NeedFlush = 2, NeedBreak = 4,
                  Blocking = 8 };
  Q_DECLARE_FLAGS(Properties, Property)
  Serial(const QString &device, Properties properties, Log::Base &base);
  virtual ~Serial() { close(); }
  virtual Type type() const { return SerialType; }
  virtual QString device() const { return _device; }

  static const QStringList &probedDeviceList();
  static IOTypes probe(const QString &device);
  static bool isAvailable() { return true; }

  bool setMode(int iflag, int cflag, Speed speed, uint readTimeout); // in ms
  virtual bool send(const char *data, uint size);
  virtual QCString receive(uint size);
  bool flush();
  bool doBreak(uint duration); // in ms
  bool setParameters(termios info);
  bool getParameters(termios &info);

  enum Pin { CD = 0, RX, TX, DTR, SG, DSR, RTS, CTS, RI, Nb_Pins };
  struct SPinData {
    IOType ioType;
    const char *label;
  };
  static const SPinData PIN_DATA[Nb_Pins];
  virtual bool setPinOn(uint pin, bool on, LogicType type);
  virtual bool readPin(uint pin, LogicType type);
  virtual QValueVector<PinData> pinData(IOType type) const;
  virtual bool isGroundPin(uint pin) const;
  virtual uint groundPin() const { return SG; }
  virtual IOType ioType(uint pin) const;

private:
  int        _fd;
  QString    _device;
  Properties _properties;
  termios    _oldInfo;

  static QStringList *_list;
  static QStringList deviceList();

  struct SpeedData {
    uint cflag, value;
  };
  static const SpeedData SPEED_DATA[Nb_Speeds];

  virtual bool internalOpen();
  virtual void internalClose();
  bool checkForInput();
};
Q_DECLARE_OPERATORS_FOR_FLAGS(Serial::Properties)

}; // namespace

#endif
