/***************************************************************************
 * Copyright (C) 2005 Lorenz Msenlechner & Matthias Kranz                 *
 *                    <icd2linux@hcilab.org>                               *
 * Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef USB_PORT_H
#define USB_PORT_H

#include "port_base.h"

struct usb_dev_handle;
struct usb_device;

namespace Port
{

class USB : public Base
{
public:
  USB(uint vendorId, uint productId, Log::Base &base);
  virtual ~USB() { close(); }
  virtual Type type() const { return USBType; }
  virtual QString device() const { return QString::null; }

  static struct usb_device *findDevice(uint vendorId, uint productId);
  static bool isAvailable();

  struct ControlMessageData {
    int type, request, value;
    const char *bytes;
  };
  bool sendControlMessage(const ControlMessageData &data);

protected:
  virtual uint interface(const struct usb_device &) const { return 0; }
  virtual int configuration() const { return -1; } // by default no specific configuration
  bool write(uint endPoint, const char *data, uint size);
  bool read(uint endPoint, char *data, uint size);

private:
  bool _needInterruptMode;
  uint _vendorId, _productId;
  usb_dev_handle *_handle;
  usb_device     *_device;
  uint _interface;

  virtual uint timeout(uint size) const; // in ms
  virtual bool internalOpen();
  virtual void internalClose();
  void setUError(const QString &message);
  void tryToDetachDriver();
};

}; // namespace

#endif
