/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "prog_group_ui.h"

#include <klocale.h>
#include <kaction.h>

#include "common/misc.h"
#include "common/gui/purl.h"
#include "common/gui/misc_gui.h"
#include "progs/base/generic_prog.h"
#include "progs/base/prog_group.h"

Programmer::AdvancedDialog::AdvancedDialog(Base &base, QWidget *parent, const char *name)
  : KDialogBase(parent, name, true, i18n("Programmer Information"), Close, Close),
    _base(base), _firmwareContainer(0)
{
  _container = new Container(this);
  setMainWidget(_container);

  QLabel *label = new QLabel(_base.group().label(), _container);
  _container->addWidget(label, 0, 0);

  if ( _base.group().properties() & HaveFirmware ) {
    _firmwareContainer = new Container(_container, Container::Sunken);
    _container->addMultiCellWidget(_firmwareContainer, 1,1, 0,1);
    PopupButton *button = createButton(i18n("Firmware"), SLOT(updateFirmware()), _firmwareContainer);
    if ( _base.group().properties() & CanUploadFirmware ) {
      KActionCollection *ac = 0;
      KAction *action = new KAction(i18n("Upload..."), "piklab_burnchip", 0, this, SLOT(uploadFirmware()), ac);
      button->appendAction(action);
    }
    _firmwareContainer->addWidget(button, 0, 0);
    label = new QLabel(i18n("Version:"), _firmwareContainer);
    _firmwareContainer->addWidget(label, 1, 0);
    _firmwareLabel = new QLabel(base.firmwareVersion().pretty(), _firmwareContainer);
    _firmwareContainer->addWidget(_firmwareLabel, 1, 1);
  }
}

PopupButton *Programmer::AdvancedDialog::createButton(const QString &text, const char *slot, QWidget *parent)
{
  PopupButton *button = new PopupButton(text, parent);
  KActionCollection *ac = 0;
  KAction *action = new KAction(i18n("Update"), "reload", 0, this, slot, ac);
  button->appendAction(action);
  return button;
}

bool Programmer::AdvancedDialog::connectProgrammer()
{
  _base.disconnectHardware();
  if ( !_base.connectHardware() ) {
    MessageBox::sorry(i18n("Could not connect programmer."), Log::Show);
    return false;
  }
  return true;
}

void Programmer::AdvancedDialog::ensureConnected()
{
  if ( _base.state()==NotConnected || _base.state()==Running ) connectProgrammer();
}

void Programmer::AdvancedDialog::updateFirmware()
{
  ::BusyCursor bc;
  ensureConnected();
  _base.readFirmware();
  updateDisplay();
}

void Programmer::AdvancedDialog::uploadFirmware()
{
  PURL url = PURL::getOpen(":open_firmware", PURL::filter(PURL::Hex), this ,i18n("Open Firmware"));
  if ( url.isEmpty() ) return;
  ::BusyCursor bc;
  if ( !connectProgrammer() ) return;
  if ( _base.uploadFirmware(url.filepath()) )
    MessageBox::information(i18n("Firmware uploaded successfully."), Log::Show);
  else MessageBox::sorry(i18n("Error uploading firmware."), Log::Show);
  updateFirmware();
}

void Programmer::AdvancedDialog::updateDisplay()
{
  _firmwareLabel->setText(_base.firmwareVersion().pretty());
}
