/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "prog_config.h"

#include <kconfig.h>
#include <kapplication.h>

#include "prog_group.h"

//----------------------------------------------------------------------------
KConfig &Programmer::Config::config(const Group &group)
{
  KConfig *conf = kapp->config();
  conf->setGroup(group.name());
  return *conf;
}

Port::Type Programmer::Config::readPortType(const Group &group)
{
  QString s = config(group).readEntry("port_group");
  Port::Type supported = Port::Nb_Types;
  for (uint i=0; i<Port::Nb_Types; i++) {
    if ( group.isPortSupported(Port::Type(i)) ) {
      if ( s==Port::DATA[i].key ) return Port::Type(i);
      supported = Port::Type(i);
    }
  }
  Q_ASSERT( supported!=Port::Nb_Types );
  return supported;
}
void Programmer::Config::writePortType(const Group &group, Port::Type portType)
{
  config(group).writeEntry("port_group", Port::DATA[portType].key);
}

QString Programmer::Config::readPortDevice(const Group &group, Port::Type portType)
{
  QString device = config(group).readEntry(QString(Port::DATA[portType].key) + "_port_device" , QString::null);
  if ( device.isNull() ) {
    QStringList list = Port::probedDeviceList(portType);
    if ( list.isEmpty() ) return QString::null;
    return list[0];
  }
  return device;
}
void Programmer::Config::writePortDevice(const Group &group, Port::Type portType, const QString &device)
{
  config(group).writeEntry(QString(Port::DATA[portType].key) + "_port_device", device);
}

QString Programmer::Config::readFirmwareDirectory(const Group &group)
{
  return config(group).readEntry("firmware_directory", QString::null);
}
void Programmer::Config::writeFirmwareDirectory(const Group &group, const QString &path)
{
  config(group).writeEntry("firmware_directory", path);
}

//----------------------------------------------------------------------------
KConfig &Programmer::Config::config()
{
  KConfig *conf = kapp->config();
  conf->setGroup("programmer");
  return *conf;
}

bool Programmer::Config::onlyProgramNonMask()
{
  return config().readBoolEntry("only_program_non_mask", true);
}
void Programmer::Config::writeOnlyProgramNonMask(bool only)
{
  config().writeEntry("only_program_non_mask", only);
}

bool Programmer::Config::onlyVerifyProgrammed()
{
  return config().readBoolEntry("only_verify_programmed", true);
}
void Programmer::Config::writeOnlyVerifyProgrammed(bool only)
{
  config().writeEntry("only_verify_programmed", only);
}

bool Programmer::Config::powerDownAfterProgramming()
{
  return config().readBoolEntry("power_down_after_programming", true);
}
void Programmer::Config::writePowerDownAfterProgramming(bool powerDown)
{
  config().writeEntry("power_down_after_programming", powerDown);
}

bool Programmer::Config::isTargetSelfPowered()
{
  return config().readBoolEntry("target_self_powered", false);
}
void Programmer::Config::writeTargetSelfPowered(bool selfPowered)
{
  config().writeEntry("target_self_powered", selfPowered);
}

void Programmer::Config::writeBlankCheckAfterErase(bool check)
{
  config().writeEntry("blank_check_after_erase", check);
}
bool Programmer::Config::blankCheckAfterErase()
{
  return config().readBoolEntry("blank_check_after_erase", false);
}

void Programmer::Config::writePreserveEeprom(bool preserve)
{
  config().writeEntry("preserve_eeprom", preserve);
}
bool Programmer::Config::preserveEeprom()
{
  return config().readBoolEntry("preserve_eeprom", false);
}
