/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PROG_GROUP_H
#define PROG_GROUP_H

class QWidget;
#include "common/group.h"
#include "port/port.h"

namespace Programmer
{
class Base;
class Hardware;
class DeviceSpecific;
class ConfigWidget;

enum Property { NoProperty = 0, Debugger = 1, CanReleaseReset = 2, HaveFirmware = 4,
                CanUploadFirmware = 8, NeedFirmwareDirectory = 16 };
Q_DECLARE_FLAGS(Properties, Property)
Q_DECLARE_OPERATORS_FOR_FLAGS(Properties)

class Group : public ::Group
{
public:
  virtual QString xmlName() const { return name(); }
  virtual Properties properties() const = 0;
  bool isDebugger() const { return ( properties() & Debugger ); }
  virtual bool isPortSupported(Port::Type type) const = 0;
  bool checkConnection(Port::Type type, const QString &portDevice) const;
  Base *createProgrammer(const Device::Data *data, Port::Type type, const QString &portDevice) const;

protected:
  virtual Base *createBase(const Device::Data *data) const = 0;
  virtual Hardware *createHardware(Base &base, Port::Type type, const QString &portDevice) const = 0;
  virtual DeviceSpecific *createDeviceSpecific(Base &base) const = 0;
};

}; // namespace

#endif
