/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "direct_mem24.h"

#include <klocale.h>

#include "common/misc.h"

uint Direct::Mem24DeviceSpecific::read_byte(bool withAck)
{
  hardware().setRead();
  uint b = 0;
  for (uint i=0; i<8; i++) {
    hardware().setPin(Clock, Low);
    Port::usleep(5+_clockDelay);
    hardware().setPin(Clock, High);
    Port::usleep(4+_clockDelay);
    b <<= 1;
    if ( hardware().readBit() ) b |= 0x1;
  }
  if ( withAck) {
    hardware().setWrite();
    hardware().setPin(Clock, Low);
    Port::usleep(4+_clockDelay);
    hardware().setPin(DataOut, Low);
    Port::usleep(1);
    hardware().setPin(Clock, High);
    Port::usleep(4+_clockDelay);
  }
  return b;
}

bool Direct::Mem24DeviceSpecific::write_byte(uint d)
{
  hardware().setWrite();
  for (uint i=0; i<8; i++) {
    hardware().setPin(Clock, Low);
    Port::usleep(4+_clockDelay);
    if ( d & 0x8 ) hardware().setPin(DataOut, High);
    else hardware().setPin(DataOut, Low);
    Port::usleep(1);
    hardware().setPin(Clock, High);
    Port::usleep(4+_clockDelay);
    d <<= 1;
  }
  // read acknowledge
  hardware().setRead();
  hardware().setPin(Clock, Low);
  Port::usleep(5+_clockDelay);
  hardware().setPin(Clock, High);
  Port::usleep(4+_clockDelay);
  if ( hardware().readBit() ) {
    log(Log::Error, i18n("Acknowledge bit incorrect"));
    return false;
  }
  hardware().setPin(Clock, Low);
  Port::usleep(4+_clockDelay);
  return true;
}

void Direct::Mem24DeviceSpecific::start()
{
  // clock low
  hardware().setWrite();
  hardware().setPin(DataOut, High);
  Port::usleep(1);
  hardware().setPin(Clock, High);
  hardware().setPin(DataOut, Low);
  Port::usleep(4+_clockDelay);
}

void Direct::Mem24DeviceSpecific::stop()
{
  // clock low
  hardware().setWrite();
  hardware().setPin(DataOut, Low);
  Port::usleep(1);
  hardware().setPin(Clock, High);
  hardware().setPin(DataOut, High);
  Port::usleep(4+_clockDelay);
}

Direct::Mem24DeviceSpecific::Mem24DeviceSpecific(::Programmer::Base &base)
  : ::Programmer::DeviceSpecific(base), PulseEngine(base)
{}

void Direct::Mem24DeviceSpecific::setPowerOn()
{
  pulseEngine("cdPw10");
}

void Direct::Mem24DeviceSpecific::setPowerOff()
{
  pulseEngine("cdpw10");
}

void Direct::Mem24DeviceSpecific::setTargetVddOn(bool on)
{
  pulseEngine(on ? "P" : "p");
}

Device::Array Direct::Mem24DeviceSpecific::doRead()
{
  // send control byte R and address 0
  start();
  if ( !write_byte(0xA0) ) return Device::Array();
  uint nb = (device().nbBitsAddress() - nbBits(device().nbBlocks()) - 1);
  uint nbBytes = nb/8 + (nb%8 ? 1 : 0);
  for (uint i=0; i<nbBytes; i++) if ( !write_byte(0x00) ) return Device::Array();

  // send control byte W and read
  start();
  if ( !write_byte(0xA1) ) return Device::Array();
  Device::Array array(device().nbBytes());
  for (uint i=0; i<array.count(); i++)
    array[i] = read_byte(i!=array.count()-1);
  stop();

  return array;
}

bool Direct::Mem24DeviceSpecific::doWrite(const Device::Array &data)
{
  // TODO
  return true;
}
