/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "icd2_group_ui.h"

#include "progs/base/gui/prog_config_widget.h"
#include "progs/base/prog_group.h"
#include "progs/icd2/icd2_prog.h"

//----------------------------------------------------------------------------
Icd2::AdvancedDialog::AdvancedDialog(ProgrammerBase &base, QWidget *parent)
  : ::Programmer::AdvancedDialog(base, parent, "icd2_advanced_dialog")
{
  uint row = _firmwareContainer->numRows();
  QLabel *label = new QLabel(i18n("Id:"), _firmwareContainer);
  _firmwareContainer->addWidget(label, row, 0);
  _firmwareIdLabel = new QLabel(_firmwareContainer);
  _firmwareContainer->addWidget(_firmwareIdLabel, row, 1);

  row = _container->numRows();
  Container *container = new Container(_container, Container::Sunken);
  _container->addMultiCellWidget(container, row,row, 0,1);
  PopupButton *button = createButton(i18n("Voltages"), SLOT(updateVoltages()), container);
  container->addWidget(button, 0, 0);
  for (uint i=0; i<Nb_VoltageTypes; i++) {
    QLabel *label = new QLabel(i18n(VOLTAGE_LABELS[i]) + ":", container);
    container->addWidget(label, 1+i, 0);
    _voltages[i] = new QLabel(container);
    container->addWidget(_voltages[i], 1+i, 1);
  }
  row++;

  container = new Container(_container, Container::Sunken);
  _container->addMultiCellWidget(container, row,row, 0,1);
  button = createButton(i18n("Self-test"), SLOT(updateSelfTest()), container);
  container->addWidget(button, 0, 0);
  for (uint i=0; i<TestData::Nb_VoltageTypes; i++) {
    QLabel *label = new QLabel(i18n(TestData::VOLTAGE_LABELS[i]) + ":", container);
    container->addWidget(label, 1+i, 0);
    _tests[i] = new QLabel(container);
    container->addWidget(_tests[i], 1+i, 1);
  }
  row++;
}

void Icd2::AdvancedDialog::updateVoltages()
{
  ::BusyCursor bc;
  ensureConnected();
  base().readVoltages();
  updateDisplay();
}

void Icd2::AdvancedDialog::updateSelfTest()
{
  ::BusyCursor bc;
  ensureConnected();
  base().selfTest(false);
  updateDisplay();
}

void Icd2::AdvancedDialog::updateDisplay()
{
  ::Programmer::AdvancedDialog::updateDisplay();
  uchar id = base().firmwareId();
  _firmwareIdLabel->setText(id==0 ? "---" : toHexLabel(id, 2));
  for (uint i=0; i<Nb_VoltageTypes; i++) {
    double v = base().voltage(VoltageType(i));
    if ( v==::Programmer::UNKNOWN_VOLTAGE ) _voltages[i]->setText("---");
    else _voltages[i]->setText(QString("%1 V").arg(v));
  }
  for (uint i=0; i<TestData::Nb_VoltageTypes; i++)
    _tests[i]->setText(base().testData().result(TestData::VoltageType(i)));
}

//----------------------------------------------------------------------------
::Programmer::ConfigWidget *Icd2::GroupUI::createConfigWidget(QWidget *parent) const
{
  return new ::Programmer::ConfigWidget(static_cast<const Group &>(group()), parent, "icd2_config_widget");
}

::Programmer::AdvancedDialog *Icd2::GroupUI::createAdvancedDialog(::Programmer::Base &base, QWidget *parent) const
{
  return new AdvancedDialog(static_cast<ProgrammerBase &>(base), parent);
}
