/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef ICD2_DEBUG_H
#define ICD2_DEBUG_H

#include "icd2_prog.h"

namespace Icd2
{
class Debugger : public ProgrammerBase
{
Q_OBJECT
public:
  Debugger(const ::Programmer::Group &group, const Pic::Data *data);
  virtual bool setBreakpoints(const QValueVector<uint> &addresses);
  virtual bool reset();
  virtual bool updateState();
  virtual uint readRegister(uint index);
  virtual bool writeRegister(uint index, uint value);
  virtual QString statusString() const;
  virtual uint pc() const;
  uint wreg() const;
  void setWreg(uint value);

private:
  Device::Array _deArray;
  uint _deStart, _deEnd;
  uint _pc, _status, _wreg;

  virtual bool internalSetupHardware();
  virtual bool programAll(const Pic::Memory &memory);
  virtual bool verifyAll(const Pic::Memory &memory, ::Programmer::VerifyActions actions);
  virtual bool internalRun();
  virtual bool internalStep();
  virtual bool internalStop();
  virtual bool initDebugging();

  bool getDebugExecutive();
  bool writeDebugExecutive();
  bool waitForTargetMode(TargetMode mode);
  Pic::Memory toDebugMemory(const Pic::Memory &memory, bool withDebugExecutive);
  bool gatherInformation();
  bool setBreakPoint(uint address);
  uint readBreakPoint();
  uint gotoInstruction(uint address) const;
  static const uint NOP;
};

//-----------------------------------------------------------------------------
class DebuggerGroup : public Group
{
public:
  virtual QString name() const { return "icd2_debugger"; }
  virtual QString label() const { return i18n("ICD2 Debugger"); }
  virtual ::Programmer::Properties properties() const { return Group::properties() | ::Programmer::Debugger; }

protected:
  virtual void addDevice(const Device::Data *data);
  virtual ::Programmer::Base *createBase(const Device::Data *data) const { return new Debugger(*this, static_cast<const Pic::Data *>(data)); }
};

}; // namespace

#endif
