/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef ICD2_PROG_H
#define ICD2_PROG_H

#include <klocale.h>

#include "progs/base/prog_group.h"
#include "devices/pic/pic_prog.h"
#include "icd2.h"

namespace Icd2
{
class Hardware;

//-----------------------------------------------------------------------------
class ProgrammerBase : public ::Programmer::PicBase
{
Q_OBJECT
public:
  ProgrammerBase(const Programmer::Group &group, const Pic::Data *data, const char *name);
  double voltage(VoltageType type) const { return _voltages[type]; }
  bool readVoltages();
  const TestData &testData() const { return _testData; }
  bool selfTest(bool ask);
  virtual bool readFirmware();
  uchar firmwareId() const { return _firmwareId; }

protected:
  Hardware &hardware();
  virtual void initConnect() { clear(); }
  virtual bool internalSetupHardware();
  virtual bool initProgramming();
  virtual bool internalRun();
  virtual bool internalStop();
  virtual bool updateState();
  virtual bool doUploadFirmware(QFile &file);

private:
  uchar    _firmwareId;
  double   _voltages[Nb_VoltageTypes];
  TestData _testData;

  void clear();
  bool checkFirmware();
};

//-----------------------------------------------------------------------------
class Programmer : public ProgrammerBase
{
Q_OBJECT
public:
  Programmer(const ::Programmer::Group &group, const Pic::Data *data);
};

//-----------------------------------------------------------------------------
class Group : public ::Programmer::Group
{
public:
  virtual QString xmlName() const { return "icd2"; }
  virtual ::Programmer::Properties properties() const;
  virtual bool isPortSupported(Port::Type type) const { return ( type==Port::SerialType || type==Port::USBType ); }

protected:
  virtual void initSupported();
  virtual ::Programmer::Hardware *createHardware(::Programmer::Base &base, Port::Type type, const QString &portDevice) const;
  virtual ::Programmer::DeviceSpecific *createDeviceSpecific(::Programmer::Base &base) const;
};

class ProgrammerGroup : public Group
{
public:
  virtual QString name() const { return "icd2"; }
  virtual QString label() const { return i18n("ICD2 Programmer"); }

protected:
  virtual ::Programmer::Base *createBase(const Device::Data *data) const { return new Programmer(*this, static_cast<const Pic::Data *>(data)); }
};

}; // namespace

#endif
