/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2002-2003 Stephen Landamore <stephen@landamore.com>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "icd2_serial.h"

#include <klocale.h>

#include "common/misc.h"

//-----------------------------------------------------------------------------
Icd2::SerialPort::SerialPort(const QString &device, Log::Manager &manager)
  : Port::Serial(device, NeedDrain | NeedFlush, manager)
{}

bool Icd2::SerialPort::open(Speed speed)
{
  if ( !Port::Serial::open() ) return false;
  // 8 bit data - enable receiver - enable hardware flow control
  if ( !setMode(0, CS8 | CREAD | CRTSCTS, speed, 100) ) return false;
  //cfsetispeed(&_info, speed);
  //cfsetospeed(&_info, speed);
  // 8 data bits
  //_info.c_cflag &= ~CSIZE;
  //_info.c_cflag |= CS8;
  //_info.c_iflag &= ~ISTRIP;
  // no parity
  //_info.c_cflag &= ~(PARENB|PARODD);
  //_info.c_iflag &= ~INPCK;
  // 1 stop bit
  //_info.c_cflag &= ~CSTOPB;
  // raw access to serial port (uncooked)
  //_info.c_iflag &= ~(IGNCR|INLCR|ICRNL|IXOFF|IXON);
  //_info.c_oflag &= ~OPOST;
  //_info.c_cflag &= ~HUPCL;
  //_info.c_cflag |= (CREAD|CRTSCTS);
  //_info.c_lflag &= ~(CLOCAL|ICANON|ECHO|ECHONL|ISIG|IEXTEN|TOSTOP);
  // misc stuff
  //_info.c_cc[VMIN]  = 0; // wait for 1 char or timeout
  //_info.c_cc[VTIME] = 0; // wait this long, then fail
   return true;
}

//-----------------------------------------------------------------------------
Icd2::SerialHardware::SerialHardware(::Programmer::Base &base, const QString &portDevice)
  : Hardware(base, new SerialPort(portDevice, base))
{}

bool Icd2::SerialHardware::internalConnect(const QCString &mode)
{
  static_cast<SerialPort *>(_port)->open(Port::Serial::S19200);
  reset();
  if ( hasError() ) return false;
  _port->send("Z", 1);
  _port->receive(4);
  if ( hasError() ) return false;
  reset();
  _port->send(mode, mode.size());
  QCString s = _port->receive(1);
  if ( hasError() ) return false;
  if ( s.upper()!=mode ) {
    log(Log::Error, i18n("Failed to set port mode to '%1'.").arg(mode));
    return false;
  }
  //log(Log::Debug, "set fast speed");
  //if ( !setFastSpeed() ) return false;
  return true;
}

void Icd2::SerialHardware::reset()
{
  static_cast<Port::Serial *>(_port)->setPinOn(Port::Serial::DTR, false, Port::PositiveLogic); // Trigger DTR to reset icd2
  Port::msleep(10);
  static_cast<Port::Serial *>(_port)->setPinOn(Port::Serial::DTR, true, Port::PositiveLogic); // remove reset
  Port::msleep(10);
}

bool Icd2::SerialHardware::setFastSpeed()
{
  if ( !command("4D") ) return false; // go faster
  static_cast<SerialPort *>(_port)->open(Port::Serial::S57600);
  Port::msleep(100); // ...we do need to delay here
  return !hasError();
}
