/***************************************************************************
 * Copyright (C) 2005 Lorenz Msenlechner & Matthias Kranz                 *
 *                    <icd2linux@hcilab.org>                               *
 * Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef ICD2_USB_H
#define ICD2_USB_H

#include "icd2.h"
#include "port/usb_port.h"

namespace Icd2
{

//-----------------------------------------------------------------------------
class USBPort : public Port::USB
{
public:
  USBPort(uint deviceId, Log::Manager &manager);

  bool connectDevice(const QCString &mode);
  virtual bool send(const char *data, uint size);
  virtual QCString receive(uint size);

private:
  uchar _seqnum;

  typedef enum SequenceType { Receive = 0, Send = 1, Connect  = 2 };
  bool doSequence(SequenceType type, char *data, uint size);
  virtual uint interface(const struct usb_device &device) const;
};

//------------------------------------------------------------------------------
class USBHardware : public Hardware
{
public:
  USBHardware(::Programmer::Base &base);

private:
  static const Port::USB::ControlMessageData CONTROL_MESSAGE_DATA[];
  enum { ID_FIRMWARE = 0x8000, // ICD2 id before usb firmware is transmitted
         ID_CLIENT = 0x8001    // ICD2 id after firmware is transmitted
  };
  virtual bool internalConnect(const QCString &mode);
};

}; // namespace

#endif
