/***************************************************************************
 * Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PICKIT_H
#define PICKIT_H

#include "progs/icd2/microchip.h"
#include "port/usb_port.h"
#include "devices/pic/pic_memory.h"
#include "devices/pic/pic_prog.h"

namespace Pickit
{
enum Mode { BootloadMode = 0, NormalMode, Nb_Modes };

//-----------------------------------------------------------------------------
class Array
{
public:
  uint length() const { return _data.count(); }
  QString pretty() const;
  uchar &operator[](uint i) { return _data[i]; }
  uchar operator[](uint i) const { return _data[i]; }

protected:
  Array(uint length);

private:
  QMemArray<uchar> _data;

  friend class USBPort;
};

//-----------------------------------------------------------------------------
class USBPort : public Port::USB
{
public:
  USBPort(uint deviceId, Log::Manager &manager);
  Array array() const { return Array(arrayLength()); }
  bool command(const Array &cmd);
  bool command(char c);
  bool command(const char *s);
  Array receive();

protected:
  virtual uint arrayLength() const = 0;
  virtual uint writeEndPoint() const = 0;
  virtual uint readEndPoint() const = 0;

private:
  virtual uint timeout(uint) const { return 10000; } // 10s
  enum { CONFIG_HID = 1, // use HID for pickit configuration
         CONFIG_VENDOR = 2 // vendor specific configuration
  };
  int configuration() const { return CONFIG_VENDOR; }
};

//-----------------------------------------------------------------------------
class Hardware : public ::Programmer::PicHardware
{
public:
  Hardware(::Programmer::Base &base, USBPort *port);
  Mode getMode(VersionData &version);
  USBPort &port() { return static_cast<USBPort &>(*_port); }
  const USBPort &port() const { return static_cast<USBPort &>(*_port); }
  QValueVector<uint> receiveWords(uint nbBytesWord);
  bool writeWords(uint max, char c, uint nbBytesWord,
                  uint &i, const Device::Array &data);

protected:
  virtual bool internalConnectHardware();
};

//-----------------------------------------------------------------------------
class DeviceSpecific : public ::Programmer::PicDeviceSpecific
{
public:
  DeviceSpecific(::Programmer::Base &base) : ::Programmer::PicDeviceSpecific(base) {}
  Hardware &hardware() { return static_cast<Hardware &>(*_base.hardware()); }
  virtual void setPowerOn();
  virtual void setPowerOff();
  virtual void setTargetVddOn(bool on);
  virtual char entryMode() const = 0;
};

//-----------------------------------------------------------------------------
class BMDeviceSpecific : public DeviceSpecific
{
public:
  BMDeviceSpecific(::Programmer::Base &base) : DeviceSpecific(base) {}
  virtual bool gotoMemory(Pic::MemoryRangeType type) = 0;
  virtual Device::Array doRead(Pic::MemoryRangeType type);
  virtual bool incrementPC(uint nb);
  virtual uint nbWrites(Pic::MemoryRangeType type) const = 0;
  virtual ::Programmer::Result doEraseCode();
};

//-----------------------------------------------------------------------------
class Baseline : public BMDeviceSpecific
{
public:
  Baseline(::Programmer::Base &base) : BMDeviceSpecific(base) {}
  virtual ::Programmer::Result doEraseEeprom() { return ::Programmer::Error; }
  virtual bool doErase(bool) { return doEraseCode(); }
  virtual bool gotoMemory(Pic::MemoryRangeType type);
  virtual bool doWrite(Pic::MemoryRangeType type, const Device::Array &data, bool force);
};

class P16F : public BMDeviceSpecific
{
public:
  P16F(::Programmer::Base &base) : BMDeviceSpecific(base) {}
  virtual bool gotoMemory(Pic::MemoryRangeType type);
  virtual bool doWrite(Pic::MemoryRangeType type, const Device::Array &data, bool force);
  virtual ::Programmer::Result doEraseEeprom();
  virtual bool doErase(bool);
};

}; // namespace

#endif
