/***************************************************************************
 * Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PICKIT2_H
#define PICKIT2_H

#include "pickit.h"

namespace Pickit2
{
  struct Data {
    char  entryMode, progMode;
    uchar progWidth;
  };
  extern const Data &data(const QString &device);

//-----------------------------------------------------------------------------
class USBPort : public Pickit::USBPort
{
public:
  USBPort(Log::Manager &manager) : Pickit::USBPort(0x0033, manager) {}

private:
  virtual uint arrayLength() const { return 64; }
  virtual uint readEndPoint() const { return 0x81; }
  virtual uint writeEndPoint() const { return 0x01; }
};

//-----------------------------------------------------------------------------
class Hardware : public Pickit::Hardware
{
public:
  Hardware(::Programmer::Base &base) : Pickit::Hardware(base, new USBPort(base)) {}
  double getVdd();
  bool setResetMode(Pic::ResetMode mode);
  Pickit::Array command(uchar cmd, uint nbBytes, uint address,
                        uint i = 0, bool longAddress = true) const;
  bool uploadFirmware(const Pic::Memory &memory);
};

//-----------------------------------------------------------------------------
class Baseline : public Pickit::Baseline
{
public:
  Baseline(::Programmer::Base &base) : Pickit::Baseline(base) {}
  virtual bool init();
  virtual char entryMode() const { return data(device().name()).entryMode; }
  virtual uint nbWrites(Pic::MemoryRangeType type) const { return (type==Pic::MemoryEeprom ? 4 : 16); }
};

//-----------------------------------------------------------------------------
class P16F : public Pickit::P16F
{
public:
  P16F(::Programmer::Base &base) : Pickit::P16F(base) {}
  virtual bool init();
  virtual char entryMode() const { return data(device().name()).entryMode; }
  virtual uint nbWrites(Pic::MemoryRangeType type) const { return (type==Pic::MemoryCode ? 16 : 4); }
};

class P16F87XA : public P16F
{
public:
  P16F87XA(::Programmer::Base &base) : P16F(base) {}
  virtual bool init();
};

class P16F7X : public P16F
{
public:
  P16F7X(::Programmer::Base &base) : P16F(base) {}
  virtual bool init();
};

class P16F716 : public P16F
{
public:
  P16F716(::Programmer::Base &base) : P16F(base) {}
  virtual bool init();
};

//-----------------------------------------------------------------------------
class P18F : public Pickit::DeviceSpecific
{
public:
  P18F(::Programmer::Base &base) : Pickit::DeviceSpecific(base) {}
  Hardware &hardware() { return static_cast<Hardware &>(Pickit::DeviceSpecific::hardware()); }
  virtual bool init();
  virtual ::Programmer::Result doEraseCode();
  virtual ::Programmer::Result doEraseEeprom();
  virtual bool doErase(bool) { return doEraseCommand(0x87, 0x0F); }
  bool doEraseCommand(uint cmd1, uint cmd2);
  virtual Device::Array doRead(Pic::MemoryRangeType type);
  virtual bool doWrite(Pic::MemoryRangeType type, const Device::Array &data, bool force);
  virtual char entryMode() const { return data(device().name()).entryMode; }
};

class P18F2X20 : public P18F
{
public:
  P18F2X20(::Programmer::Base &base) : P18F(base) {}
  virtual ::Programmer::Result doEraseCode();
  virtual ::Programmer::Result doEraseEeprom();
  virtual bool doErase(bool) { return doEraseCommand(0x80); }
  bool doEraseCommand(uint cmd);
};

}; // namespace

#endif
