/***************************************************************************
 * Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "pickit2_prog.h"

#include <klocale.h>

#include "devices/base/device_list.h"
#include "pickit2.h"

//----------------------------------------------------------------------------
bool Pickit2::Base::internalSetResetMode(Pic::ResetMode mode)
{
  return static_cast<Hardware &>(hardware()).setResetMode(mode);
}

bool Pickit2::Base::internalSetupHardware()
{
  setTargetVddOn(false);
  //_vdd = static_cast<Hardware &>(hardware()).getVdd();
  //log(Log::NormalDebug, i18n("  Target Vdd = %1V").arg(_vdd));
  return Pickit::Base::internalSetupHardware();
}

bool Pickit2::Base::doUploadFirmware(QFile &file)
{
  const Device::Data &data = *Device::lister().data("18F2550");
  Pic::Memory memory(static_cast<const Pic::Data &>(data));
  QString error;
  QStringList warnings;
  Pic::Memory::WarningTypes warningTypes;
  if ( !memory.load(file, error, warningTypes, warnings) ) {
    log(Log::Error, i18n("Could not read firmware file \"%1\" (%2).").arg(file.name()).arg(error));
    return false;
  }
  if ( warningTypes!=Pic::Memory::NoWarning ) {
    log(Log::Error, i18n("Firmware hex file seems incompatible with device 18F2550 inside PICkit2."));
    return false;
  }
  if ( !static_cast<Hardware &>(hardware()).uploadFirmware(memory) ) {
    log(Log::Error, i18n("Failed to upload firmware."));
    return false;
  }
  log(Log::Information, i18n("  Firmware succesfully uploaded."));
  return true;
}

//----------------------------------------------------------------------------
Programmer::Hardware *Pickit2::Group::createHardware(Programmer::Base &base, Port::Type, const QString &) const
{
  return new Hardware(base);
}
