/***************************************************************************
 * Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PICKIT2_PROG_H
#define PICKIT2_PROG_H

#include <klocale.h>

#include "pickit_prog.h"

namespace Pickit2
{
//----------------------------------------------------------------------------
class Base : public Pickit::Base
{
Q_OBJECT
public:
  Base(const Programmer::Group &group, const Pic::Data *data) : Pickit::Base(group, data) {}

private:
  virtual bool internalSetupHardware();
  virtual VersionData minVersion() const { return VersionData(1, 10, 0); }
  virtual VersionData maxVersion() const { return VersionData(1, 10, 0); }
  virtual bool internalSetResetMode(Pic::ResetMode mode);
  virtual bool doUploadFirmware(QFile &file);
};

//----------------------------------------------------------------------------
class Group : public Pickit::Group
{
public:
  virtual QString name() const { return "pickit2"; }
  virtual QString label() const { return i18n("PICkit2"); }
  virtual Programmer::Properties properties() const { return ::Programmer::HaveFirmware | ::Programmer::CanUploadFirmware; }

protected:
  virtual void initSupported();
  virtual Programmer::Base *createBase(const Device::Data *data) const { return new Base(*this, static_cast<const Pic::Data *>(data)); }
  virtual Programmer::Hardware *createHardware(Programmer::Base &base, Port::Type type, const QString &portDevice) const;
  virtual Programmer::DeviceSpecific *createDeviceSpecific(Programmer::Base &base) const;
};

}; // namespace

#endif
