/***************************************************************************
 * Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "pickit_prog.h"

#include <klocale.h>

#include "pickit.h"
#include "progs/base/prog_config.h"

Pickit::Base::Base(const Programmer::Group &group, const Pic::Data *data)
  : ::Programmer::PicBase(group, data, "pickit_programmer")
{}

Pickit::Hardware &Pickit::Base::hardware()
{
  return static_cast<Hardware &>(*_hardware);
}

bool Pickit::Base::readFirmware()
{
  Mode mode = hardware().getMode(_firmwareVersion);
  switch (mode) {
    case Nb_Modes: Q_ASSERT(false); return false;
    case BootloadMode:
      log(Log::Information, i18n("Bootloader firmware version is %1").arg(_firmwareVersion.pretty()));
      break;
    case NormalMode: {
      log(Log::Information, i18n("Firmware version is %1").arg(_firmwareVersion.pretty()));
      VersionData vd(_firmwareVersion.major(), _firmwareVersion.mid(), 0); // ignore minor
      if ( vd.toUInt()<minVersion().toUInt() )
        log(Log::Warning, i18n("Firmware version (%1) is lower than the version required to program some devices (%2). Please upgrade.")
                          .arg(_firmwareVersion.pretty()).arg(minVersion().pretty()));
      if ( maxVersion().toUInt()<vd.toUInt() )
        log(Log::Warning, i18n("Firmware version (%1) is higher than the tested version (%2). You may experience problems...")
                          .arg(_firmwareVersion.pretty()).arg(maxVersion().pretty()));
      break;
    }
  }
  return !hasError();
}

bool Pickit::Base::internalSetupHardware()
{
  if (_specific) {
    if ( !_specific->init() ) return false;
    _specific->setTargetVddOn(false);
  }
  readFirmware();
  if (_specific) {
    bool selfPowered = ::Programmer::Config::isTargetSelfPowered();
    log(Log::Information, i18n("  Target is self-powered: %1").arg(selfPowered ? "true" : "false"));
    _specific->setTargetVddOn(!selfPowered);
  }
  return true;
}
