/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003 Alain Gibaud <alain.gibaud@free.fr>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PROJECT_H
#define PROJECT_H

#include <qdom.h>

#include "common/gui/purl.h"

//----------------------------------------------------------------------------
class XmlDataFile
{
public:
  XmlDataFile(const PURL &url, const QString &name);
  virtual ~XmlDataFile() {}
  PURL url() const { return _url; }
  virtual bool load(QString &error);
  bool save(QString &error) const;

  QString value(const QString &group, const QString &key, const QString &defValue) const;
  void setValue(const QString &group, const QString &key, const QString &value);
  QStringList listValues(const QString &group, const QString &key) const;
  void appendListValue(const QString &group, const QString &key, const QString &value);
  void removeListValue(const QString &group, const QString &key, const QString &value);
  void clearList(const QString &group, const QString &key);

protected:
  PURL _url;

private:
  QString      _name;
  QDomDocument _document;

  QDomElement findChildElement(QDomElement element, const QString &tag) const;
  QDomElement createChildElement(QDomElement element, const QString &tag);
  void removeChilds(QDomNode node) const;
};

//----------------------------------------------------------------------------
class Project : public XmlDataFile
{
public:
  Project(const PURL &url) : XmlDataFile(url, "piklab") {}
  virtual bool load(QString &error);

  PDirectory directory() const { return url().directory(); }
  QString name() const { return url().basename(); }
  PURL::List absoluteFiles() const;
  PURL hexUrl() const { return url().toFileType(PURL::Hex); }
  QString version() const;
  QString description() const;
  PURL::List openedFiles() const;
  QStringList objectsForLinker(const QString &extension = QString::null) const;
  QStringList librariesForLinker(const QString &prefix = QString::null) const;

  void removeFile(const PURL &url); // take absolute filepath (but inside project dir)
  void addFile(const PURL &url); // take absolute filePath (but inside project dir)
  void setVersion(const QString &version);
  void setDescription(const QString &description);
  void setOpenedFiles(const PURL::List &list);
};

#endif
