/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "register_view.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <klocale.h>

#include "devices/base/gui/device_group_ui.h"
#include "progs/base/prog_list.h"
#include "tools/misc_gui.h"
#include "toplevel.h"

//-----------------------------------------------------------------------------
const char * const Register::LISTVIEW_LABEL = I18N_NOOP("Registers");

void Register::showView()
{
  ::BusyCursor bc;
  Editor *e = MainWindow::editorManager().findEditor(LISTVIEW_LABEL);
  if ( e==0 ) {
    e = new View(i18n(LISTVIEW_LABEL), LISTVIEW_LABEL);
    static_cast<View *>(e)->setDevice(MainWindow::device());
    MainWindow::editorManager().addEditor(e);
  }
  MainWindow::editorManager().showEditor(e);
}

//-----------------------------------------------------------------------------
Register::View::View(const QString &title, const QString &tag, QWidget *parent)
  : DeviceEditor(NoCapability, title, tag, parent, "register_view"), _debugging(false)
{
  MainWindow::debugManager().addEditor(this);
}

void Register::View::setDevice(const QString &device, bool force)
{
  bool oldDebugging = _debugging;
  _debugging = MainWindow::programmerGroup().isDebugger();
  DeviceEditor::setDevice(device, force || oldDebugging!=_debugging);
}

QWidget *Register::View::createView(const Device::Data *data, QWidget *parent)
{
  if ( data==0 ) return new QWidget(parent);
  QWidget *w = data->groupui().createRegisterView(*data, _debugging, parent);
  if (w) {
    connect(w, SIGNAL(readSignal(uint)), SIGNAL(readSignal(uint)));
    connect(w, SIGNAL(writeSignal(uint, uint)), SIGNAL(writeSignal(uint, uint)));
    connect(w, SIGNAL(setWatchedSignal(uint, bool)), SIGNAL(setWatchedSignal(uint, bool)));
  }
  return w;
}

void Register::View::readAll()
{
  MainWindow::debugManager().readAll();
}

void Register::View::stopWatchAll()
{
  MainWindow::debugManager().stopWatchAll();
}
