/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *   Copyright (C) 2004 Alain Gibaud <alain.gibaud@free.fr>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef CONFIG_WIDGET_H
#define CONFIG_WIDGET_H

#include <qcombobox.h>
#include <qwidget.h>
#include <qpixmap.h>
#include <qlayout.h>
#include <qtabwidget.h>
#include <qvaluevector.h>

#include "common/gui/misc_gui.h"
#include "tools/misc_gui.h"
#include "tools/tool.h"

//-----------------------------------------------------------------------------
class ConfigWidget : public QWidget
{
Q_OBJECT
public:
  ConfigWidget(QWidget *parent = 0, const char *name = 0)
    : QWidget(parent, name) {}
  virtual void loadConfig() {}
  virtual QString title() const { return QString::null; }
  virtual QString header() const { return QString::null; }
  virtual QPixmap pixmap() const { return QPixmap(); }

public slots:
  virtual void saveConfig() {}
};

//-----------------------------------------------------------------------------
class GridConfigWidget : public ConfigWidget
{
Q_OBJECT
public:
  GridConfigWidget(QWidget *parent = 0, const char *name = 0);

protected:
  QGridLayout *_topLayout;
};

//-----------------------------------------------------------------------------
class TabConfigWidget : public ConfigWidget
{
Q_OBJECT
public:
  TabConfigWidget(QWidget *parent = 0, const char *name = 0);

protected:
  QTabWidget *_tabWidget;
};

//-----------------------------------------------------------------------------
class ToolConfigWidget : public GridConfigWidget
{
Q_OBJECT
public:
  ToolConfigWidget(Project *project, const char *name = 0);
  virtual ~ToolConfigWidget();
  virtual void loadConfig();

public slots:
  virtual void saveConfig();

protected:
  Container     *_container;
  Project       *_project;
  QLineEdit     *_customOptions;
  SelectDirectoryWidget *_includeDirs;
  QComboBox     *_hexFormat;

  SelectDirectoryWidget * createDirectoryEntry(const QString &label);
  SelectDirectoryWidget * createIncludeDirectoryEntry() { return createDirectoryEntry(i18n("Include directories:")); }
  QLineEdit *createCustomOptionsEntry();
  QComboBox *createHexFormatEntry();
  Compile::Config &config() { return *_config; }

private:
  Compile::Config *_config;
  Tool::Category   _category;

  friend class Tool::Group;
};

//-----------------------------------------------------------------------------
class ToolsConfigWidget : public GridConfigWidget
{
Q_OBJECT
public:
  ToolsConfigWidget(Project *project, QWidget *parent);
  virtual void loadConfig();

public slots:
  virtual void saveConfig();

private slots:
  void toolChanged();

private:
  Project    *_project;
  QComboBox  *_tool;
  QTabWidget *_tabWidget;
  QValueVector<QWidgetStack *> _stacks;
};

#endif
