/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "gputils_config_widget.h"

#include <qlabel.h>
#include <qtooltip.h>
#include <klocale.h>

#include "gputils_config.h"
#include "project.h"

//----------------------------------------------------------------------------
GPUtils::AssemblerConfigWidget::AssemblerConfigWidget(Project *project)
  : ToolConfigWidget(project, "gputils_assembler_config_widget")
{
  uint row = _container->numRows();
  QLabel *label = new QLabel(i18n("Warning level:"), _container);
  _container->addWidget(label, row, 0);
  _gpasmWarning = new QComboBox(_container);
  for (uint i=0; i<GPUtils::Config::Nb_WarningLevels; i++)
    _gpasmWarning->insertItem(i18n(GPUtils::Config::WARNING_LEVEL_LABELS[i]));
  _gpasmWarning->insertItem(i18n("as in LIST directive"));
  _container->addWidget(_gpasmWarning, row, 1);
  _includeDirs = createIncludeDirectoryEntry();
  _hexFormat = (project==0 ? createHexFormatEntry() : 0);
}

void GPUtils::AssemblerConfigWidget::loadConfig()
{
  ToolConfigWidget::loadConfig();
  _gpasmWarning->setCurrentItem(static_cast<Config &>(config()).gpasmWarningLevel());
}

void GPUtils::AssemblerConfigWidget::saveConfig()
{
  ToolConfigWidget::saveConfig();
  static_cast<Config &>(config()).setGPAsmWarningLevel(_gpasmWarning->currentItem());
}

//----------------------------------------------------------------------------
GPUtils::DisassemblerConfigWidget::DisassemblerConfigWidget()
  : ToolConfigWidget(0, "gputils_disassembler_config_widget")
{
  uint row = _container->numRows();
  _gpdasmShortOutput = new QCheckBox(i18n("Plain source code"), _container);
  QToolTip::add(_gpdasmShortOutput ,i18n("Output a source file reassemblable by gpasm."));
  _container->addMultiCellWidget(_gpdasmShortOutput, row,row, 0,1);
}

void GPUtils::DisassemblerConfigWidget::loadConfig()
{
  ToolConfigWidget::loadConfig();
  _gpdasmShortOutput->setChecked(static_cast<Config &>(config()).outputShortFormat());
}

void GPUtils::DisassemblerConfigWidget::saveConfig()
{
  ToolConfigWidget::saveConfig();
  static_cast<Config &>(config()).writeOutputShortFormat(_gpdasmShortOutput->isChecked());
}

//----------------------------------------------------------------------------
GPUtils::ProjectLinkerConfigWidget::ProjectLinkerConfigWidget(Project &project)
  : ToolConfigWidget(&project, "gputils_project_linker_config_widget")
{
  _hexFormat = createHexFormatEntry();
}
