/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 1992-2003 Trolltech AS.                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "misc_gui.h"

#include <qpushbutton.h>
#include <qapplication.h>
#include <qtimer.h>
#include <qwidgetstack.h>
#include <qtabwidget.h>
#include <qobjectlist.h>
#include <qpainter.h>
#include <qheader.h>
#include <qmetaobject.h>

#include <kcursor.h>
#include <krun.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kmessagebox.h>

#include "global_config.h"

//-----------------------------------------------------------------------------
UrlLabel::UrlLabel(const QString &url, const QString &text,
                   QWidget *parent, const char *name)
  : KURLLabel(url, text, parent, name)
{
  connect(this, SIGNAL(leftClickedURL()), SLOT(urlClickedSlot()));
}

void UrlLabel::urlClickedSlot()
{
  (void)new KRun(url());
}

//-----------------------------------------------------------------------------
Splitter::Splitter(const QValueList<int> &defaultSizes, Orientation o, QWidget *parent, const char *name)
  : QSplitter(o, parent, name), _defaultSizes(defaultSizes)
{
  Q_ASSERT(name);
  setOpaqueResize(true);
  QTimer::singleShot(0, this, SLOT(updateSizes()));
}

Splitter::~Splitter()
{
  GuiConfig::config().writeEntry(QString(name()) + "_sizes", sizes());
}

void Splitter::updateSizes()
{
  QValueList<int> sizes = GuiConfig::config().readIntListEntry(QString(name()) + "_sizes");
  for (uint i=sizes.count(); i<_defaultSizes.count(); i++) sizes.append(_defaultSizes[i]);
  setSizes(sizes);
}

//-----------------------------------------------------------------------------
Dialog::Dialog(const QSize &defaultSize, QWidget *parent, const char *name, bool modal,
               const QString &caption, int buttonMask, ButtonCode defaultButton, bool separator)
  : KDialogBase(parent, name, modal, caption, buttonMask, defaultButton, separator),
    _defaultSize(defaultSize)
{
  Q_ASSERT(name);
  QWidget *main = new QWidget(this);
  setMainWidget(main);

  QTimer::singleShot(0, this, SLOT(updateSize()));
}

Dialog::Dialog(const QSize &defaultSize, DialogType type,
               const QString &caption, int buttonMask, ButtonCode defaultButton,
               QWidget *parent, const char *name, bool modal, bool separator)
  : KDialogBase(type, caption, buttonMask, defaultButton, parent, name, modal, separator),
    _defaultSize(defaultSize)
{
  Q_ASSERT(name);
  QTimer::singleShot(0, this, SLOT(updateSize()));
}

Dialog::~Dialog()
{
  GuiConfig::config().writeEntry(QString(name()) + "_size", size());
}

void Dialog::updateSize()
{
  resize(GuiConfig::config().readSizeEntry(QString(name()) + "_size", &_defaultSize));
}

//-----------------------------------------------------------------------------
SelectDirectoryWidget::SelectDirectoryWidget(const PDirectory &defaultDir,
                                             QWidget *parent, const char *name)
  : QWidget(parent, name), _defaultDir(defaultDir)
{
  QHBoxLayout *top = new QHBoxLayout(this, 0, 10);

  _edit = new QLineEdit(this);
  connect(_edit, SIGNAL(textChanged(const QString &)), SIGNAL(changed()));
  top->addWidget(_edit);
  KIconLoader loader;
  QIconSet iconset = loader.loadIcon("fileopen", KIcon::Toolbar);
  QPushButton *button = new  QPushButton(iconset, QString::null, this);
  connect(button, SIGNAL(clicked()), SLOT(buttonClicked()));
  top->addWidget(button);
}

void SelectDirectoryWidget::buttonClicked()
{
  PDirectory dir = PDirectory::getExisting(_defaultDir.path(), this, i18n("Select directory"));
  if ( dir.isEmpty() ) return;
  _edit->setText(dir.path());
  emit changed();
}
