/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef MISC_GUI_H
#define MISC_GUI_H

#include <qlayout.h>
#include <qlineedit.h>
#include <qsplitter.h>
#include <qvaluevector.h>

#include <klocale.h>
#include <kurllabel.h>
#include <kdialogbase.h>

#include "common/gui/purl.h"
#include "common/log.h"

//-----------------------------------------------------------------------------
class UrlLabel : public KURLLabel
{
Q_OBJECT
public:
  UrlLabel(const QString &url, const QString &text,
           QWidget *parent = 0, const char *name = 0);

private slots:
  void urlClickedSlot();
};

//-----------------------------------------------------------------------------
class Splitter : public QSplitter
{
Q_OBJECT
public:
  Splitter(const QValueList<int> &defaultSizes, Orientation orientation,
           QWidget *parent, const char *name);
  virtual ~Splitter();

private slots:
  void updateSizes();

private:
  QValueList<int> _defaultSizes;
};

//-----------------------------------------------------------------------------
class Dialog : public KDialogBase
{
Q_OBJECT
public:
  Dialog(const QSize &defaultSize, QWidget *parent, const char *name, bool modal,
         const QString &caption, int buttonMask, ButtonCode defaultButton, bool separator = false);
  Dialog(const QSize &defaultSize, DialogType type, const QString &caption,
         int buttonMask, ButtonCode defaultButton, QWidget *parent, const char *name,
         bool modal, bool separator = false);
  virtual ~Dialog();

private slots:
  void updateSize();

private:
  QSize _defaultSize;
};

//-----------------------------------------------------------------------------
class SelectDirectoryWidget : public QWidget
{
Q_OBJECT
public:
  SelectDirectoryWidget(const PDirectory &defaultDir, QWidget *parent = 0, const char *name = 0);
  void setDirectories(const QString &dir) { _edit->setText(dir); }
  QString directories() const { return _edit->text(); }

signals:
  void changed();

private slots:
  void buttonClicked();

private:
  const PDirectory _defaultDir;
  QLineEdit *_edit;
};

#endif
