/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "picc.h"

#include <qregexp.h>
#include "tools/picc_compile.h"
#include "tools/picc_config_widget.h"
#include "tools/picc_config.h"
#include "devices/pic/pic_memory.h"
#include "devices/base/device_list.h"

//----------------------------------------------------------------------------
bool PICC::Base::checkExecutable(const QString &exec, bool withWine) const
{
  return !run(exec, "--ver", withWine).isNull();
}

void PICC::Group::initSupported()
{
  const Tool::Base *b = base(Tool::Compiler);
  QString s = b->run(b->executable(), "--CHIPINFO", withWine());
  if ( s.isNull() ) return;
  QStringList lines = QStringList::split('\n', s);
  for (uint i=0; i<lines.count(); i++) {
    QRegExp re("([A-Za-z0-9]+):.*");
    if ( !re.exactMatch(lines[i]) ) continue;
    const Device::Data *data = Device::lister().data(re.cap(1));
    if (data) addDevice(data);
  }
}

bool PICC::Group::needs(Tool::ActionType type, Tool::Category category) const
{
  return ( type==Tool::Project && category!=Tool::BinToHex );
}

Compile::Process *PICC::Group::processFactory(Tool::Category category, const Project *project) const
{
  switch (category) {
    case Tool::Compiler:
      if (project) return new PICC::CompileProjectFile;
      return new PICC::CompileStandaloneFile;
    case Tool::Assembler:
      if (project) return new PICC::AssembleProjectFile;
      return new PICC::AssembleStandaloneFile;
    case Tool::Linker:
      if (project) return new PICC::LinkProject;
      break;
    case Tool::BinToHex: break;
    case Tool::ObjectViewer: break;
    case Tool::Disassembler: break;
    case Tool::Nb_Categories: break;
  }
  Q_ASSERT(false);
  return 0;
}

ToolConfigWidget *PICC::Group::configWidgetFactory(Tool::Category category, Project *project) const
{
  switch (category) {
    case Tool::Compiler:  return new PICC::ConfigWidget(Tool::Compiler, project);
    case Tool::Assembler: return new PICC::ConfigWidget(Tool::Assembler, project);
    case Tool::Linker:    return new PICC::ConfigWidget(Tool::Linker, project);
    case Tool::BinToHex: break;
    case Tool::ObjectViewer: break;
    case Tool::Disassembler: break;
    case Tool::Nb_Categories: Q_ASSERT(false); break;
  }
  return 0;
}

Compile::Config *PICC::Group::configFactory(::Project *project) const
{
  return new Config(project);
}

PURL::FileType PICC::Group::sourceFileType(PURL::SourceType type) const
{
  switch (type) {
    case PURL::Asm: return PURL::AsmPICC;
    case PURL::C:   return PURL::CSource;
    case PURL::Nb_SourceTypes: break;
  }
  Q_ASSERT(false);
  return PURL::Nb_FileTypes;
}

QString PICC::Group::generateDeviceConfig(PURL::SourceType, const Device::Memory &memory, bool &ok) const
{
  ok = true;
  return "Config generation not implemented yet";
}

Tool::Base *PICC::Group::baseFactory(Tool::Category c, const char *exec) const
{
  switch (c) {
    case Tool::Compiler:      return new Base(exec);
    case Tool::Assembler:     return new Base(exec);
    case Tool::Linker:        return new Base(exec);
    case Tool::ObjectViewer:  return 0;
    case Tool::BinToHex:      return 0;
    case Tool::Disassembler:  return 0;
    case Tool::Nb_Categories: break;
  }
  Q_ASSERT(false);
  return 0;
}

//----------------------------------------------------------------------------
QString PICC::PICCLiteGroup::downloadText() const
{
  return i18n("<qt><a href=\"http://www.htsoft.com\">PICC-Lite</a> is a freeware C compiler distributed by HTSoft.</qt>");
}

//----------------------------------------------------------------------------
QString PICC::PICCGroup::downloadText() const
{
  return i18n("<qt><a href=\"http://www.htsoft.com\">PICC</a> is a C compiler distributed by HTSoft.</qt>");
}

//----------------------------------------------------------------------------
QString PICC::PICC18Group::downloadText() const
{
  return i18n("<qt><a href=\"http://www.htsoft.com\">PICC 18</a> is a C compiler distributed by HTSoft.</qt>");
}
