/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PICC_H
#define PICC_H

#include "tool.h"

namespace PICC
{
//----------------------------------------------------------------------------
class Base : public Tool::Base
{
public:
  Base(const char *exec) : Tool::Base(exec) {}

private:
  virtual bool checkExecutable(const QString &exec, bool withWine) const;
};

class Group : public Tool::Group
{
public:
  virtual PDirectory linkerScriptsDirectory() const { return PDirectory(); }
  virtual PURL linkerScriptUrl(Compile::LinkType, const QString &) const { return PURL(); }
  virtual PDirectory headerFilesDirectory() const { return PDirectory(); }
  virtual bool needs(Tool::ActionType, Tool::Category) const;
  virtual PURL::FileType sourceFileType(PURL::SourceType type) const;
  virtual QString generateDeviceConfig(PURL::SourceType type, const Device::Memory &memory, bool &ok) const;

protected:
  Tool::Base *baseFactory(Tool::Category category, const char *exec) const;

private:
  virtual void initSupported();
  virtual Compile::Process *processFactory(Tool::Category category, const ::Project *project) const;
  virtual Compile::Config *configFactory(::Project *project) const;
  virtual ToolConfigWidget *configWidgetFactory(Tool::Category category, ::Project *project) const;
};

//----------------------------------------------------------------------------
class PICCLiteGroup : public Group
{
public:
  virtual QString name() const { return "picclite"; }
  virtual QString label() const { return i18n("PICC Lite Compiler"); }
  virtual QString downloadText() const;

private:
  virtual Tool::Base *baseFactory(Tool::Category category) const { return Group::baseFactory(category, "picl"); }
};

//----------------------------------------------------------------------------
class PICCGroup : public Group
{
public:
  virtual QString name() const { return "picc"; }
  virtual QString label() const { return i18n("PICC Compiler"); }
  virtual QString downloadText() const;

private:
  virtual Tool::Base *baseFactory(Tool::Category category) const { return Group::baseFactory(category, "picc"); }
};

//----------------------------------------------------------------------------
class PICC18Group : public Group
{
public:
  virtual QString name() const { return "picc18"; }
  virtual QString label() const { return i18n("PICC-18 Compiler"); }
  virtual QString downloadText() const;

private:
  virtual Tool::Base *baseFactory(Tool::Category category) const { return Group::baseFactory(category, "picc18"); }
};

}; // namespace

#endif
