/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "picc_compile.h"

#include <qregexp.h>
#include <klocale.h>

#include "tools/process.h"
#include "tools/picc_config.h"
#include "project.h"

using namespace Compile;

//-----------------------------------------------------------------------------
QStringList PICC::Process::arguments() const
{
  QStringList args = Compile::Process::arguments();
  args += "--ERRFORMAT";
  args += "--WARNFORMAT";
  args += "--MSGFORMAT";
  args += "--CHIP=" + data().device;
  return args;
}

void PICC::Process::logStderrLine(const QString &line)
{
  // #### TODO
  emit log(Log::Data, line);
}

//-----------------------------------------------------------------------------
QStringList PICC::CompileStandaloneFile::arguments() const
{
  QStringList args = Process::arguments();
  args += "-M" + data().item.url.toFileType(PURL::Map).filename();
  args += "-G" + data().item.url.toFileType(PURL::Sym).filename();
  args += "--ASMLIST";
  args += config()->includeDirs(Tool::Compiler, directory(), "-I");
  args += config()->customOptions(Tool::Compiler, directory());
  args += data().item.url.filename();
  return args;
}

FileData::List PICC::CompileStandaloneFile::files() const
{
  FileData::List list;
  list.append(FileData(data().item.url.toFileType(PURL::Lst), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Map), Generated | InProject));
  list.append(FileData(data().item.url.toExtension("obj"), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Hex), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Sym), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Sdb), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Hxl), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Coff), Generated/* | InProject*/));
  list.append(FileData(data().item.url.toFileType(PURL::Rlf), Generated));
  return list;
}

//-----------------------------------------------------------------------------
QStringList PICC::CompileProjectFile::arguments() const
{
  QStringList args = Process::arguments();
  args += "-S";
  args += config()->includeDirs(Tool::Compiler, data().project->directory(), "-I");
  args += config()->customOptions(Tool::Compiler, data().project->directory());
  args += data().item.url.relativeTo(directory());
  return args;
}

FileData::List PICC::CompileProjectFile::files() const
{
  FileData::List list;
  list.append(FileData(data().item.url.toFileType(PURL::AsmPICC), Generated | InProject));
  return list;
}

//-----------------------------------------------------------------------------
QStringList PICC::AssembleStandaloneFile::arguments() const
{
  QStringList args;
  args += "-M" + data().item.url.toFileType(PURL::Map).filename();
  args += "-G" + data().item.url.toFileType(PURL::Sym).filename();
  args += "--ASMLIST";
  args += config()->includeDirs(Tool::Assembler, directory(), "-I");
  args += config()->customOptions(Tool::Assembler, directory());
  args += data().item.url.filename();
  return args;
}

FileData::List PICC::AssembleStandaloneFile::files() const
{
  FileData::List list;
  list.append(FileData(data().item.url.toFileType(PURL::Lst), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Map), Generated | InProject));
  list.append(FileData(data().item.url.toExtension("obj"), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Hex), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Sym), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Sdb), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Hxl), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Rlf), Generated));
  return list;
}

//-----------------------------------------------------------------------------
QStringList PICC::AssembleProjectFile::arguments() const
{
  QStringList args = Process::arguments();
  args += "-C";
  args += "--ASMLIST";
  args += config()->includeDirs(Tool::Assembler, data().project->directory(), "-I");
  args += config()->customOptions(Tool::Assembler, data().project->directory());
  args += data().item.url.relativeTo(directory());
  return args;
}

FileData::List PICC::AssembleProjectFile::files() const
{
  FileData::List list;
  list.append(FileData(data().item.url.toExtension("obj"), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Lst), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Rlf), Generated));
  return list;
}

//-----------------------------------------------------------------------------
QStringList PICC::LinkProject::arguments() const
{
  QStringList args = Process::arguments();
  args += "-O" + data().project->hexUrl().filename();
  args += "-M" + data().item.url.toFileType(PURL::Map).filename();
  args += "-G" + data().item.url.toFileType(PURL::Sym).filename();
  if ( data().linkType==LinkIcd2 ) args += "--DEBUGGER=icd2";
  args += config()->customOptions(Tool::Linker, data().project->directory());
  args += data().project->objectsForLinker("obj");
  args += data().project->librariesForLinker();
  return args;
}

FileData::List PICC::LinkProject::files() const
{
  FileData::List list;
  list.append(FileData(data().item.url.toFileType(PURL::Map), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Hex), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Sym), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Coff), Generated/* | InProject*/));
  list.append(FileData(data().item.url.toFileType(PURL::Sdb), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Hxl), Generated));
  return list;
}
