/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef SDCC_H
#define SDCC_H

#include "tool.h"

namespace SDCC
{
  extern QString toDeviceName(const QString &device);
  enum Family { P14 = 0, P16, Nb_Families };
  struct FamilyData {
    const char *name;
  };
  extern const FamilyData FAMILY_DATA[Nb_Families];
  extern Family family(const QString &device);

class Compiler : public Tool::Base
{
public:
  Compiler() : Tool::Base("sdcc") {}

protected:
  virtual bool checkExecutable(const QString &exec, bool withWine) const;
};

class Group : public Tool::Group
{
public:
  virtual QString name() const { return "sdcc"; }
  virtual QString label() const { return i18n("Small Device C Compiler"); }
  virtual QString downloadText() const;
  virtual PDirectory linkerScriptsDirectory() const { return PDirectory(); }
  virtual PURL linkerScriptUrl(Compile::LinkType, const QString &) const { return PURL(); }
  virtual PDirectory headerFilesDirectory() const { return PDirectory(); }
  virtual bool needs(Tool::ActionType type, Tool::Category category) const;
  virtual PURL::FileType sourceFileType(PURL::SourceType type) const;
  virtual QString generateDeviceConfig(PURL::SourceType type, const Device::Memory &memory, bool &ok) const;

private:
  virtual const Tool::Group *group(Tool::Category category) const;
  virtual Tool::Base *baseFactory(Tool::Category category) const;
  virtual void initSupported();
  virtual Compile::Process *processFactory(Tool::Category category, const ::Project *project) const;
  virtual ToolConfigWidget *configWidgetFactory(Tool::Category category, ::Project *project) const;
  virtual Compile::Config *configFactory(::Project *project) const;
};

}; // namespace

#endif
