/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "sdcc_compile.h"

#include <qregexp.h>
#include <qapplication.h>
#include <kmessagebox.h>
#include <klocale.h>

#include "tools/process.h"
#include "tools/sdcc.h"
#include "project.h"
#include "sdcc_config.h"
#include "gputils_compile.h"

using namespace Compile;

//-----------------------------------------------------------------------------
void SDCC::CompileFile::logStderrLine(const QString &line)
{
  QRegExp re("([^:]*):([0-9]+):([^:]+):(.+)");
  if ( re.exactMatch(line) ) {
    if ( re.cap(1)=="-" ) {
      emit log(Log::Error, re.cap(4).stripWhiteSpace());
      return;
    }
    PURL url(re.cap(1));
    if ( !url.isInto(directory()) ) url = PURL(directory(), re.cap(1));
    QString type = re.cap(3).stripWhiteSpace();
    emit log(Log::Error, url.filename() + ":" + re.cap(2) + ": "
             + re.cap(4).stripWhiteSpace(), url, re.cap(2).toUInt()-1);
    return;
  }
  bool isDebug;
  Log::Type type = GPUtils::LinkProject::logType(line, isDebug);
  if (isDebug) emit log(Log::NormalDebug, line);
  else emit log(type, line);
}

//-----------------------------------------------------------------------------
QStringList SDCC::CompileStandaloneFile::arguments() const
{
  QStringList args;
  args += QString("-m") + FAMILY_DATA[family(data().device)].name;
  args += QString("-") + toDeviceName(data().device);
  args += "-V"; // verbose
  args += QString("-Wl") + "-o" + data().item.url.toFileType(PURL::Hex).filepath();
  args += QString("-Wl") + "-m"; // output map file
  HexBuffer::Format format = config()->hexFormat();
  if( format!=HexBuffer::Nb_Formats )
    args += QString("-Wl") + "-a" + HexBuffer::FORMATS[format];
  args += config()->includeDirs(Tool::Compiler, directory(), "-I");
  args += config()->customOptions(Tool::Compiler, directory());
  args += data().item.url.filepath();
  return args;
}

FileData::List SDCC::CompileStandaloneFile::files() const
{
  FileData::List list;
  list.append(FileData(data().item.url.toFileType(PURL::Lst), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Map), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Object), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::AsmGPAsm), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Hex), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Rst), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Sym), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Mem), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Adb), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Lnk), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::Cod), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::SDCC_P), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::SDCC_D), Generated));
  return list;
}

//-----------------------------------------------------------------------------
QStringList SDCC::CompileProjectFile::arguments() const
{
  QStringList args;
  args += "-S"; // compile only
  args += "-V"; // verbose
  args += QString("-m") + FAMILY_DATA[family(data().device)].name;
  args += QString("-") + toDeviceName(data().device);
  args += config()->includeDirs(Tool::Compiler, data().project->directory(), "-I");
  args += config()->customOptions(Tool::Compiler, data().project->directory());
  args += data().item.url.relativeTo(directory());
  return args;
}

FileData::List SDCC::CompileProjectFile::files() const
{
  FileData::List list;
  list.append(FileData(data().item.url.toFileType(PURL::Object), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::AsmGPAsm), Generated | InProject));
  list.append(FileData(data().item.url.toFileType(PURL::Adb), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::SDCC_P), Generated));
  list.append(FileData(data().item.url.toFileType(PURL::SDCC_D), Generated));
  return list;
}
