/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef TOOL_H
#define TOOL_H

#include <qstringlist.h>
#include <qmap.h>
#include <klocale.h>

#include "common/gui/purl.h"
#include "common/group.h"
class Project;
class ToolConfigWidget;
namespace Device {
  class Data;
  class Memory;
}

namespace Compile
{
  enum LinkType { LinkNormal, LinkIcd2 };
  class Process;
  class Config;

  class TodoItem {
  public:
    TodoItem() {}
    TodoItem(const PURL &_url, bool _generated) : url(_url), generated(_generated) {}
    PURL url;
    bool generated;
  };

  class Data {
  public:
    Data() {}
    Data(const TodoItem &i, const QString &d, const Project *p, LinkType lt = LinkNormal)
      : item(i), device(d), project(p), linkType(lt) {}
    TodoItem item;
    QString  device;
    const Project *project;
    LinkType linkType;
  };
};

namespace Tool
{
  enum Category { Compiler = 0, Assembler, Linker, BinToHex, ObjectViewer,
                  Disassembler, Nb_Categories };
  struct CategoryData {
    const char *name, *label;
  };
  extern const CategoryData CATEGORY_DATA[Nb_Categories];
  enum ActionType { StandaloneFile, Project };
  class Group;

//-----------------------------------------------------------------------------
class Base
{
public:
  Base(const char *executable) : _executable(executable) {}
  virtual ~Base() {}
  const char *baseExecutable() const { return _executable; }
  QString executable() const;
  virtual bool checkExecutable(const QString &exec, bool withWine) const = 0;
  bool check(const QString &device = QString::null) const;
  QString run(const QString &exec, const QStringList &options, bool withWine) const;

protected:
  Category     _category;
  const Group *_group;
  const char  *_executable;

  friend class Group;
};

//-----------------------------------------------------------------------------
class Group : public ::Group
{
public:
  virtual void init();
  const Base *base(Category category) const { return _bases[category]; }
  virtual QString downloadText() const = 0;
  bool withWine() const;
  virtual PDirectory linkerScriptsDirectory() const = 0;
  virtual PURL linkerScriptUrl(Compile::LinkType type, const QString &device) const = 0;
  virtual PDirectory headerFilesDirectory() const = 0;
  virtual bool needs(ActionType type, Category category) const = 0;
  Compile::Process *createCompileProcess(Category category, const ::Project *project) const;
  ToolConfigWidget *createConfigWidget(Category category, ::Project *project) const;
  Compile::Config *createConfig(::Project *project) const;
  bool supports(PURL::SourceType type) const;
  virtual PURL::FileType sourceFileType(PURL::SourceType type) const = 0;
  virtual QString generateDeviceConfig(PURL::SourceType type, const Device::Memory &memory, bool &ok) const = 0;
  virtual Compile::Process *processFactory(Category category, const ::Project *project) const = 0;
  virtual ToolConfigWidget *configWidgetFactory(Category category, ::Project *project) const = 0;

protected:
  virtual const Group *group(Category) const { return this; }
  virtual Base *baseFactory(Category category) const = 0;
  virtual Compile::Config  *configFactory(::Project *project) const = 0;

private:
  QMap<QString, const Device::Data *> _supported;
  Base *_bases[Nb_Categories];
};

class Lister : public ::Lister<Group>
{
public:
  Lister();
};

extern const Lister &lister();

}; // namespace

#endif
