/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef TOOL_CONFIG_H
#define TOOL_CONFIG_H

#include "common/gui/purl.h"
#include "tools/tool.h"
#include "devices/base/hex_buffer.h"
class Project;

//----------------------------------------------------------------------------
namespace Compile
{
class Config
{
public:
  Config(Project *project) : _group(0), _project(project) {}
  virtual ~Config() {}
  QString rawIncludeDirs(Tool::Category category) const;
  QStringList includeDirs(Tool::Category category, const PDirectory &dir, const QString &prefix) const;
  void setRawIncludeDirs(Tool::Category category, const QString &dirs);
  QString rawCustomOptions(Tool::Category category) const;
  QStringList customOptions(Tool::Category category, const PDirectory &dir) const;
  void setRawCustomOptions(Tool::Category category, const QString &options);
  HexBuffer::Format hexFormat() const;
  void setHexFormat(HexBuffer::Format format);
  QString device() const { return device(_project); }
  void setDevice(const QString &device) { setDevice(_project, device); }
  uint toolType() const { return toolType(_project); }
  void setToolType(uint i) { setToolType(_project, i); }
  uint warningLevel(Tool::Category category) const { return uintValue(category, "warning_level", 0); }
  void setWarningLevel(Tool::Category category, uint level) { setValue(category, "warning_level", QString::number(level)); }
  PDirectory executableDirectory() const { return PDirectory(value(Tool::Compiler, "executable_path", QString::null)); }
  void setExecutableDirectory(const PDirectory &dir);
  bool withWine() const;
  void setWithWine(bool withWine);

  static uint toolType(const Project *project);
  static void setToolType(Project *project, uint i) { setGlobalValue(project, "tool", Tool::lister().group(i).name()); }
  static QString device(const Project *project);
  static void setDevice(Project *project, const QString &device) { setGlobalValue(project, "device", device); }

  static QString globalValue(const Project *project, const QString &key, const QString &defaultValue);
  static void setGlobalValue(Project *project, const QString &key, const QString &value);

protected:
  void setValue(Tool::Category category, const QString &key, const QString &value);
  QString value(Tool::Category category, const QString &key, const QString &defaultValue) const;
  void setValue(Tool::Category category, const QString &key, bool value) { setValue(category, key, QString(value ? "true" : "false")); }
  bool boolValue(Tool::Category category, const QString &key, bool defaultValue) const;
  void setValue(Tool::Category category, const QString &key, uint value) { setValue(category, key, QString::number(value)); }
  bool uintValue(Tool::Category category, const QString &key, uint defaultValue) const;
  virtual bool defaultWithWine() const = 0;

private:
  const Tool::Group *_group;
  Project           *_project;

  friend class Tool::Group;

  static QString value(const Project &project, const QString &subGroup, const QString &key, const QString &defaultValue);
  static void setValue(Project &project, const QString &subGroup, const QString &key, const QString &value);
  static QString value(const QString &group, const QString &subGroup, const QString &key, const QString &defaultValue);
  static void setValue(const QString &group, const QString &subGroup, const QString &key, const QString &value);
};

}; // namespace

#endif
