/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PIKMAIN_H
#define PIKMAIN_H

class QLabel;
class QProgressBar;
class QTabWidget;
#include "common/qflags.h"
#include <kmainwindow.h>
#include <kurl.h>

#include "editor_manager.h"
#include "project_manager.h"
#include "progs/base/generic_prog.h"
#include "debug_manager.h"
#include "config_center.h"

class HexEditor;
class Editor;
class Project;
namespace Log { class Widget; };
namespace Compile {
  class LogWidget;
  class FileData;
  class Manager;
};
class ConfigGenerator;
namespace Watch { class View; };

class MainWindow : public KMainWindow
{
Q_OBJECT
public:
  MainWindow();
  enum State { Idle, Compiling, Programming, Closing };
  State state() const { return _state; }
  bool initProgramming();
  void endProgramming();

  static MainWindow &self() { return *_self; }
  static const Programmer::Group &programmerGroup();
  static Programmer::Base *programmer() { return _self->_programmer; }
  static KAction &action(const char *name);
  static QPopupMenu &popup(const char *name);
  static EditorManager &editorManager() { return *_self->_editorManager; }
  static Editor *currentEditor() { return _self->_editorManager->currentEditor(); }
  static QString device();
  static const Device::Data *deviceData();
  static DebugManager &debugManager() { return *_self->_debugManager; }
  static ProjectManager::View &projectManager() { return *_self->_projectManager; }
  static Watch::View *watchView() { return _self->_watchView; }

public slots:
  void newSourceFile();
  void newProject();
  void openProject();
  void buildProject();
  void cleanBuild();
  void configure(ConfigCenter::Type showType = ConfigCenter::General);
  void configureProject();

private slots:
    void newHexFile();
    bool openFile();
    void openRecentFile(const KURL &url);

    void compileFile();
    void compileSuccess();
    void compileFailure();
    void updateFile(const Compile::FileData &data);
    void stopBuild();

    void program();
    void verify();
    void read();
    void erase();
    void blankCheck();
    void connectDevice();
    void toggleDevicePower();
    void disconnectDevice();
    void setTotalSteps(uint total);
    void addToTotalSteps(uint add);
    void addProgress(uint steps);

    void progRun();
    void progStop();
    void progRestart();
    void progAdvanced();
    void debugRun();
    void debugHalt();
    void debugReset();
    void debugStep();
    void clearBreakpoints();

  void applyToolbarSettings();
  void configureToolbar();
  void configureEditor();
  void configureProgrammer() { configure(ConfigCenter::ProgSelect); }
  void configureTool();

  void runPikloops();
  void deviceInfo();
  void configGenerator();

  void openRecentProject(const KURL &url);
  void closeProject();

  void initialLoading();
  void updateGUI() { setState(_state); }

signals:
  void stateChanged();

private:
  static MainWindow *_self;

private:
  EditorManager        *_editorManager;
  QTabWidget           *_logTab;
  Log::Widget          *_programLog;
  Compile::LogWidget   *_compileLog;
  QLabel               *_actionStatus, *_debugStatus, *_editorStatus;
  QPushButton          *_programmerStatus, *_toolStatus;
  QProgressBar         *_actionProgress;
  Compile::Manager     *_compileManager;
  DebugManager         *_debugManager;
  ConfigGenerator      *_configGenerator;
  QTabWidget           *_tabWidget;
  ProjectManager::View *_projectManager;
  Watch::View          *_watchView;
  Programmer::Base     *_programmer;
  State _state;

  void saveSettings();
  HexEditor *getHexEditor();
  void setState(State state);
  virtual bool queryClose();
  Project *project() { return (_projectManager ?_projectManager->project() : 0); }
  const Project *project() const { return (_projectManager ? _projectManager->project() : 0); }
  void stopOperations();
  void cleanBuild(bool singleFile);
  virtual void keyPressEvent(QKeyEvent *e);
  void createProgrammer(const Device::Data *data);
};

#endif
