/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "watch_view.h"

#include <qheader.h>
#include <qcombobox.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kiconloader.h>

#include "toplevel.h"
#include "register_view.h"
#include "devices/base/gui/device_group_ui.h"

Watch::View::View(QWidget *parent)
  : ListView(parent, "watch_view"), _popup(0)
{
  _data = MainWindow::deviceData();
  connect(this, SIGNAL(mouseButtonClicked(int, QListViewItem *, const QPoint &, int)),
          SLOT(itemClicked(int, QListViewItem *, const QPoint &, int)));
  connect(this, SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)),
          SLOT(contextMenu(QListViewItem *, const QPoint &, int)));
  header()->hide();
  setSorting(-1);
  addColumn(QString::null);
  addColumn(QString::null);
  addColumn(QString::null);
  setFullWidth(true);
  setRootIsDecorated(false);
  setAllColumnsShowFocus(true);

  init();
}

void Watch::View::init()
{
  clear();
  if ( _data==0 ) return;
  delete _popup;
  _popup = _data->groupui().createWatchPopup(*_data, this);
  _rootItem = new KListViewItem(this, i18n(Register::LISTVIEW_LABEL));
  _rootItem->setSelectable(false);
  _rootItem->setOpen(true);
}

void Watch::View::setWatched(uint i, bool watched)
{
  QListViewItem *child;
  for (child=_rootItem->firstChild(); child; child=child->nextSibling()) {
    if ( static_cast<Device::RegisterItem *>(child)->index()!=i ) continue;
    if ( !watched ) delete child;
    return;
  }
  Q_ASSERT(watched);
  Device::RegisterItem *item = _data->groupui().createRegisterItem(*_data, i, _rootItem);
  item->init();
  QListViewItem *last;
  for (last=_rootItem->firstChild(); last->nextSibling(); last=last->nextSibling());
  if (last) item->moveItem(last);
}

void Watch::View::setRegisterValue(uint i, uint value)
{
  QListViewItem *child;
  for (child=_rootItem->firstChild(); child; child=child->nextSibling()) {
    Device::RegisterItem *ri = static_cast<Device::RegisterItem *>(child);
    if ( ri->index()!=i ) continue;
    ri->setValue(value);
  }
}

void Watch::View::itemClicked(int button, QListViewItem *item, const QPoint &, int)
{
  if ( button!=LeftButton ) return;
  if ( item!=_rootItem ) return;
  Register::showView();
}

void Watch::View::contextMenu(QListViewItem *item, const QPoint &p, int)
{
  if ( item==0 || _popup==0 ) return;

  if ( item==_rootItem ) {
    int res = _popup->exec(p);
    if ( res>=0 ) MainWindow::debugManager().setRegisterWatched(res, true);
  } else {
    Device::RegisterItem *ri = static_cast<Device::RegisterItem *>(item);
    KIconLoader loader;
    QPixmap deleteIcon = loader.loadIcon("editdelete", KIcon::Small);
    KPopupMenu *pop = new KPopupMenu;
    pop->insertTitle(item->text(0));
    pop->insertItem(deleteIcon, i18n("Stop Watching"), 1);
    if ( pop->exec(p)==1 ) MainWindow::debugManager().setRegisterWatched(ri->index(), false);
    delete pop;
  }
}

void Watch::View::stopWatchAll()
{
  MainWindow::debugManager().stopWatchAll();
}
